def TestOpenStuff():
    """    """
    import ioflo.base.storing as storing


    storing.Store.Clear() #clear registry
    s1 = ServerTask(store = storing.Store())
    s2 = ServerTask(store = storing.Store())

    print(s1.server.reopen())
    print(s2.server.reopen())


def Test(verbose = False):
    """Module self test



    """
    import ioflo.base.storing as storing
    import ioflo.base.tasking as tasking

    storing.Store.Clear() #clear registry
    tasking.Tasker.Clear()

    s = Server(store = storing.Store())

    s.store.expose()

    print("ready to go")
    status = s.start()

    while (not (status == STOPPED or status == ABORTED)):
        try:
            status = s.run()

        except KeyboardInterrupt: #CNTL-C shutdown skedder
            print("    Keyboard Interrupt manual shutdown of taskers ...")
            s.server.close()


            break



if __name__ == "__main__":
    Test()




"""goaling.py goal action module

"""
#print("module {0}".format(__name__))

import time
import struct

from collections import deque
try:
    from itertools import izip
except ImportError: # python3
    izip = zip

import inspect


from ..aid.sixing import *
from .globaling import *
from ..aid.odicting import odict
from ..aid import aiding
from . import excepting
from . import registering
from . import storing
from . import acting

from .consoling import getConsole
console = getConsole()

#Class definitions

class Goal(acting.Actor):
    """Goal Class for setting configuration or command value in data share """
    Registry = odict()


class GoalDirect(Goal):
    """GoalDirect Goal """

    def __init__(self, **kw):
        """Initialization method for instance. """
        super(GoalDirect, self).__init__(**kw)  #.goal inited here


    def _resolve(self, destination, destinationFields, sourceData, **kwa):
        parms = super(GoalDirect, self)._resolve( **kwa)

        destination = self._resolvePath(ipath=destination,  warn=True) # now a share
        sourceFields = sourceData.keys()
        destinationFields = self._prepareDstFields(sourceFields,
                                          destination,
                                          destinationFields)

        dstData = odict()
        for dstField, srcField in izip(destinationFields, sourceFields):
            dstData[dstField] = sourceData[srcField]

        parms['destination'] = destination
        parms['data'] = dstData
        return parms


    def action(self, destination, data, **kw):
        """
        Set destination goal to data dictionary
        parameters:
              destination = share of goal
              data = dict of data fields to assign to goal share
        """
        console.profuse("Set {0} to {1}\n".format(destination.name, data))
        destination.update(data)
        return None

class GoalIndirect(Goal):
    """GoalIndirect Goal """

    def __init__(self, **kw):
        """Initialization method for instance."""

        super(GoalIndirect, self).__init__(**kw)  #.goal inited here

    def _resolve(self, destination, destinationFields, source, sourceFields, **kwa):
        parms = super(GoalIndirect, self)._resolve( **kwa)

        destination = self._resolvePath(ipath=destination,  warn=True) # now a share
        source = self._resolvePath(ipath=source,  warn=True) # now a share

        sourceFields, destinationFields = self._prepareSrcDstFields(source,
                                                        sourceFields,
                                                        destination,
                                                        destinationFields)

        parms['destination'] = destination
        parms['destinationFields'] = destinationFields
        parms['source'] = source
        parms['sourceFields'] = sourceFields
        return parms

    def action(self, destination, destinationFields, source, sourceFields, **kw):
        """
        Set destinationFields in destination goal from sourceFields in source

        parameters:
              destination = share of goal
              source = share of source to get data from
              fields = fields to use to update goal
        """
        console.profuse("Set {0} in {1} from {2} in {3}\n".format(
                destination.name, destinationFields, source.name, sourceFields))
        data = odict()
        for gf, sf in izip(destinationFields, sourceFields):
            data[gf] = source[sf]
        destination.update(data)
        return None



"""
Metaclass and utility classes

"""
from __future__ import absolute_import, division, print_function

from collections import Iterable, Sequence
from abc import ABCMeta

# import ioflo modules
from .sixing import *

from ..base.consoling import getConsole
console = getConsole()


def metaclassify(metaclass):
    """
    Class decorator for creating a class with a metaclass.
    This enables the same syntax to work in both python2 and python3
    python3 does not support
        class name(object):
            __metaclass__ mymetaclass
    python2 does not support
        class name(metaclass=mymetaclass):

    Borrowed from six.py add_metaclass decorator

    Usage:
    @metaclassify(Meta)
    class MyClass(object):
        pass
    That code produces a class equivalent to:

    on Python 3
    class MyClass(object, metaclass=Meta):
        pass

    on Python 2
    class MyClass(object):
        __metaclass__ = MyMeta
    """
    def wrapper(cls):
        originals = cls.__dict__.copy()
        originals.pop('__dict__', None)
        originals.pop('__weakref__', None)
        for slots_var in originals.get('__slots__', ()):
            originals.pop(slots_var)
        return metaclass(cls.__name__, cls.__bases__, originals)
    return wrapper

@metaclassify(ABCMeta)
class NonStringIterable:
    """ Allows isinstance check for iterable that is not a string
    """
    #__metaclass__ = ABCMeta

    @classmethod
    def __subclasshook__(cls, C):
        if cls is NonStringIterable:
            if (not issubclass(C, basestring) and issubclass(C, Iterable)):
                return True
        return NotImplemented

@metaclassify(ABCMeta)
class NonStringSequence:
    """ Allows isinstance check for sequence that is not a string
    """
    #__metaclass__ = ABCMeta

    @classmethod
    def __subclasshook__(cls, C):
        if cls is NonStringSequence:
            if (not issubclass(C, basestring) and issubclass(C, Sequence)):
                return True
        return NotImplemented

def nonStringIterable(obj):
    """
    Returns True if obj is non-string iterable, False otherwise

    Future proof way that is compatible with both Python3 and Python2 to check
    for non string iterables.
    Assumes in Python3 that, basestring = (str, bytes)

    Faster way that is less future proof
    return (hasattr(x, '__iter__') and not isinstance(x, basestring))
    """
    return (not isinstance(obj, basestring) and isinstance(obj, Iterable))

def nonStringSequence(obj):
    """
    Returns True if obj is non-string sequence, False otherwise

    Future proof way that is compatible with both Python3 and Python2 to check
    for non string sequences.
    Assumes in Python3 that, basestring = (str, bytes)
    """
    return (not isinstance(obj, basestring) and isinstance(obj, Sequence) )




"""arbiting.py arbiter deed module

"""
#print("module {0}".format(__name__))

import time
import struct
from collections import deque
import inspect

from ..aid.sixing import *
from ..aid.odicting import odict
from .globaling import *

from . import doing

from .consoling import getConsole
console = getConsole()


#Arbiter Class
class Arbiter(doing.Doer):
    """Arbiter Doer Class
       Generic Arbiter Class for Arbiters should be subclassed

    """
    def __init__(self, output, group, inputs, **kw):
        """Initialize Instance

           Arbiters use three types of inputs to do the arbitration:
              selections must evaluate to boolean True of False but using Python rules so
                 could be any type rule is
                    if sel: then the input is selected
                    otherwise input is ignored
              confidences are used either for threshold or for weighting output
                 confidences can be number in interval [ 0.0, 1.0] or Boolean or None.
                 when necessary arbiter will convert confidence to be in [0.0, 1.0] using rules
                    if None then assumed = 1.0,
                    if True then 1.0
                    if False then 0.0
                    otherwise hard limit and convert numbers to interval [0.0, 1.0]
              importances are assumed to be in interval [0.0, 1.0] but will still work properly
                 if non negative numbers

              Values are not constrained for Switch and Priority arbiters but must be numbers for
                 Weighted arbiter.
                 With a weighted arbiter if one of the input values is not number then
                    the weighted arbiter uses the default outputs.

           Init Parameters:

           output is path name of share for output/truth of arbiter

           group is path name of group in store, group has following subgroups or shares:
              group.default = share of default value and confidence (truth) for arbiter
              group.inimps = data structure of input importances
              group.insels = data structure of input selections


           inputs is ordered dictionary (odict) of inputs:
              key = selection tag name for input
              value = tuple of input data
                 (share path name, selection importance)
                    tag is user friendly short name for input used to change selection or importance

           instance attributes
           .output = reference to output share
           .group = copy of group path name
           .default = reference to group.default value and conf share
           .insels = reference to group.insels share
           .inimps = reference to group.inimps share
           .inputs = ordered dictionary (odict) of tags and input shares

           inherited instance attributes:
           .name
           .store

        """
        super(Arbiter,self).__init__(**kw)

        #create it if it does not already exist
        self.output = self.store.create(output).create(value = 0.0)
        #updateJointly
        self.output.value = 0.0 #force value
        self.output.truth = 0.0 #force truth
        self.group = group

        #create default value conf and assign defaults if not already assigned
        self.default = self.store.create(group + '.default').create(value = 0.0)

        #check to make sure default truth in [0.0, 1.0]
        if not self.GoodTruth(self.default.truth):
            self.default.truth = self.FixTruth(self.default.truth)

        #create insels and inimps if not exist
        self.insels = self.store.create(group + '.insels')
        self.inimps = self.store.create(group + '.inimps')

        #initialize inputs, inimps and insels
        self.inputs = odict()

        for tag, stuff in inputs.items():
            input, sel, imp = stuff #break out tuple inputpath, selection, importance
            if tag: #filter out '' tags, not allowed
                input = self.store.create(input) #create share reference from path name
                self.inputs[tag] = input #save it
                self.insels.create(**{tag : sel}) #initialize selection if not exist
                self.inimps.create(**{tag : imp}) #initialize importance if not exist


    def _expose(self):
        """prints out arbiter parameters

        """
        msg = "Arbiter %s\n" % self.name
        msg += "   group = %s  outval = %s outcnf = %s defval = %s defcnf = %s\n" %\
              (self.group, self.output.value, self.output.truth, self.default.value, self.default.truth)
        msg += "   inputs = \n"
        for tag, input in self.inputs.items():
            msg += "      tag = %s, input = %s, sel = %s, imp = %s,\n" %\
                  (tag, input.name, self.insels.fetch(tag), self.inimps.fetch(tag))
            msg += "      input value = %s input truth = %s\n" % (input.value, input.truth)
        console.terse(msg)

    def update(self):
        """update should be overridden by subclass

        """
        pass


    def action(self, **kw):
        """action is to update arbiter

        """
        console.profuse("Updating arbiter {0}\n".format(self.name))
        self.update()


    @staticmethod
    def GoodTruth(truth):
        """Check if truth float in range [0.0, 1.0]
           this so only FixTruth when needed for initing default confidence
        """
        if isinstance(truth, float) and (truth >= 0.0) and (truth <= 1.0):
            return True

        return False

    @staticmethod
    def FixTruth(truth):
        """make truth to be float in range [0.0, 1.0]
           if truth is None or Boolean True then make 1.0
           If truth is Boolean False then make 0.0
           otherwise Truth assumed to be number so hard limit to interval [0.0, 1.0]
        """
        if truth is None or truth is True: #None or Boolean True
            truth = 1.0
        elif truth is False: #Boolean False
            truth = 0.0
        else:
            truth = float(min(1.0,max(0.0,truth)))

        return truth

#Switch Arbiter Class
class ArbiterSwitch(Arbiter):
    """ArbiterSwitch Arbiter Deed Class


    """
    def update(self,stamp = None):
        """update switch arbiter algorithm
           simply switch selected input to output
           ignore importances
           ignore threshold on confidence

           find first input whose:
              selection is logically True
              if found then output's value/truth is found input's value/truth

           otherwise output's value/truth is default's value/truth

        """
        for tag, input in self.inputs.items():
            if self.insels.fetch(tag): #first one that is selected
                #self.output.updateJointly(value = input.value, truth = input.truth, stamp = stamp)
                self.output.value = input.value
                self.output.truth = input.truth
                self.output.stamp = stamp
                return

        #self.output.updateJointly(value = self.default.value,
        #                           truth = self.default.truth, stamp = stamp)
        self.output.value = self.default.value
        self.output.truth = self.default.truth
        self.output.stamp = stamp

#Priority Arbiter Class
class ArbiterPriority(Arbiter):
    """ArbiterPriority Arbiter Deed Class


    """
    def update(self,stamp = None):
        """update priority arbiter algorithm

           an input is selected if its selection is logically true
           an input is sufficient if its confidence is > default truth
           confidence constrained to range [0.0, 1.0]
           if an input's truth is True or None then use truth = 1.0
           an input is most important if it has maximum importance of all selected sufficient inputs

           find first and most important input

           if found then output's value/truth is found input's value/truth
           else output's value/truth is default's value/truth

        """
        #find max selected input
        inputmax = None
        impmax = 0.0
        truthmax = 0.0
        for tag, input in self.inputs.items():
            truth = self.FixTruth(input.truth) #fix truth to be float in range [0.0, 1.0]
            imp = self.inimps.fetch(tag)  # or could use form self.inimps[tag]
            if (  self.insels.fetch(tag) and
                  (truth > self.default.truth) and
                  (imp > impmax) ):
                inputmax = input
                impmax = imp
                truthmax = truth

        if inputmax:
            #self.output.updateJointly(value = inputmax.value, truth = truthmax, stamp = stamp)
            self.output.value = inputmax.value
            self.output.truth = truthmax
            self.output.stamp = stamp
        else:
            #self.output.updateJointly(value = self.default.value,
            #                              truth = self.default.truth, stamp = stamp)
            self.output.value = self.default.value
            self.output.truth = self.default.truth
            self.output.stamp = stamp

#Trusted Arbiter Class
class ArbiterTrusted(Arbiter):
    """ArbiterTrusted Arbiter Deed Class


    """
    def update(self,stamp = None):
        """update trusted arbiter algorithm

           an input is selected if its selection is logically true
           an input is sufficient if its confidence is > default truth
           input's truth confidence constrained to range [0.0, 1.0]
           if an input's truth is True or None then use truth = 1.0
           an input has the highest confidence if its confidence is maximum amoung all
              selected sufficient inputs
           an input is most important if it has maximum importance of all highest confidence inputs

           find first most important input

           if found then output's value/truth is found input's value/truth
           else output's value/truth is default's value/truth

        """
        #find max selected input
        inputmax = None
        impmax = 0.0
        truthmax = 0.0
        for tag, input in self.inputs.items():
            sel = self.insels.fetch(tag) #or could use form self.insel[tag]
            truth = self.FixTruth(input.truth) #fix truth to be float in range [0.0, 1.0]
            imp = self.inimps.fetch(tag)
            if (sel and (truth > self.default.truth) ):
                if (truth > truthmax):
                    truthmax = truth
                    impmax = imp
                    inputmax = input
                elif ( (truth == truthmax) and (imp > imputmax) ):
                    truthmax = truth
                    impmax = imp
                    inputmax = input

        if inputmax:
            #self.output.updateJointly(value = inputmax.value, truth = truthmax, stamp = stamp)
            self.output.value = inputmax.value
            self.output.truth = truthmax
            self.output.stamp = stamp
        else:
            #self.output.updateJointly(value = self.default.value,
            #                           truth = self.default.truth, stamp = stamp)
            self.output.value = self.default.value
            self.output.truth = self.default.truth
            self.output.stamp = stamp

#Weighted Arbiter Class
class ArbiterWeighted(Arbiter):
    """ArbiterWeighted Arbiter Deed Class


    """
    def update(self,stamp = None):
        """update weighted arbiter algorithm

           an input is selected if its selection is logically true
           compute normalized weighted average of all selected inputs where average is given by
           weighted conf = Sum(imp * cnf)/Sum(imp)
           weighted value = Sum(imp * cnf * value)/Sum(imp * cnf)

           input truth values are fixed up to be float in range [0.0, 1.0]

           if weighted conf > default truth then
              output's value/truth is found input's value/fixed truth
           else
              output's value/truth is default's value/truth

        """
        wgtval = 0.0
        wgtcnf = 0.0
        wgtimp = 0.0
        try:
            for tag, input in self.inputs.items():
                if self.insels.fetch(tag): #or could use form self.insel[tag]
                    truth = self.FixTruth(input.truth)
                    imp = self.inimps.fetch(tag) #or could use form self.insel[tag]
                    wgtimp += imp
                    wgtcnf += imp * truth
                    wgtval += imp * truth * input.value

            wgtval = wgtval / float(wgtcnf)
            wgtcnf = wgtcnf / float(wgtimp)

        except TypeError:#one of the input values is not number
            console.terse("     Warning, bad input value for Arbiter {0}\n".format(self.name))
            #self.output.updateJointly(value = self.default.value,
            #                              truth = self.default.truth, stamp = stamp)
            self.output.value = self.default.value
            self.output.truth = self.default.truth
            self.output.stamp = stamp
            return
        except ZeroDivisionError:  #no selected input with non zero truth or importance
            #self.output.updateJointly(value = self.default.value,
            #                           truth = self.default.truth, stamp = stamp)
            self.output.value = self.default.value
            self.output.truth = self.default.truth
            self.output.stamp = stamp
            return


        if wgtcnf > self.default.truth:
            #self.output.updateJointly(value = wgtval, truth = wgtcnf, stamp = stamp)
            self.output.value = wgtval
            self.output.truth = wgtcnf
            self.output.stamp = stamp

        else:
            #self.output.updateJointly(value = self.default.value,
            #                           truth = self.default.truth, stamp = stamp)
            self.output.value = self.default.value
            self.output.truth = self.default.truth
            self.output.stamp = stamp




"""acting.py action module

"""
#print("module {0}".format(__name__))

import time
import struct
from collections import deque, Mapping
from functools import wraps
import inspect
import copy
try:
    from itertools import izip
except ImportError: #python 3 zip is same as izip
    izip = zip

from ..aid.sixing import *
from .globaling import *
from ..aid.odicting import odict

from ..aid.metaing import metaclassify, nonStringIterable
from ..aid.aiding import nameToPath
from . import excepting
from . import registering
from . import storing
from . import framing

from .consoling import getConsole
console = getConsole()


#Class definitions

class Act(object):
    """ Container class for actor resolve time initialization and runtime operation """
    __slots__ = ('frame', 'context', 'act', 'actor',
                 'registrar', 'inits', 'ioinits', 'parms',
                 'prerefs', 'human', 'count')

    def __init__(self, frame=None, context=None, act=None, actor=None,
                 registrar=None, inits=None, ioinits=None, parms=None,
                 prerefs=None, human='', count=0, **kwa ):
        """ Initialization method for instance.

            Attributes:
            .frame = ref to Frame holding this Act
            .context = name string of actioning context set when added to frame
            .act = ref to super Act if self is embedded in another Act such as Need
            .actor = ref to Actor instance or actor name to be resolved
                    call performs instances method action
            .registrar = ref to Actor class holding .Registry
            .parms = dictionary of keyword arguments for Actor instance call
            .inits = dictionary of arguments to Actor.__init__()
            .ioinits = dictionary of arguments to Actor._initio()
            .prerefs = dictionary of init, ioinit, and parm refs to resolve
            .human = human friendly version of action declaration
            .count = line count in floscript of action declaration
        """
        super(Act,self).__init__(**kwa) # in case of MRO

        self.frame = frame
        self.context = context
        self.act = act # should normally not be set until resolve time
        self.actor = actor #callable instance performs action
        self.registrar = registrar or Actor
        self.parms = parms if parms is not None else odict() # parms must always be not None
        self.inits = inits if inits else None # store None if empty dict
        self.ioinits = ioinits if ioinits else None # store None if empty dict
        self.prerefs = prerefs if prerefs else None # store None if empty dict
        self.human = human  # human readable version of FloScript declaration
        self.count = count  # line number or count of FloScript declaration

    def clone(self):
        """ Return clone of self in support of framer cloning
            clones is dict whose items keys are original framer names
            and values are duples of (original,clone) framer references
        """
        if isinstance(self.frame, framing.Frame):
            msg = ("CloneError: Attempting to clone resolved frame link"
                "  '{0}'.".format(self.frame.name))
            raise excepting.CloneError(msg)

        if self.act:
            msg = ("CloneError: Attempting to clone resolved act link"
                   "  '{0}'.".format(self.act))
            raise excepting.CloneError(msg)

        if isinstance(self.actor, Actor):
            msg = ("CloneError: Attempting to clone resolved actor"
                "  '{0}'.".format(self.actor.name))
            raise excepting.CloneError(msg)

        # frame and context for Act set when add Act to frame later
        clone = copy.deepcopy(self)
        return clone

    def __call__(self): #make Act instance callable as function
        """ Define act as callable object """
        return (self.actor(**self.parms))

    def expose(self):
        """ Show attributes"""
        console.terse("Act Actor {0} Parms {1} in Frame {2} Context {3} SuperAct {4}\n".format(
                    self.actor, self.parms, self.frame, self.context, self.act))
        if self.actor:
            self.actor._expose()

    def resolve(self, **kwa):
        """ Resolve .frame attribute and .actor. Cause .actor to resolve its parms """
        if self.act: # this is sub act of another act self.act so get super act context
            self.frame = self.act.frame
            self.context = self.act.context
        self.frame = framing.resolveFrame(self.frame,
                                          who="Act for {0}".format(self.actor),
                                          desc="act's",
                                          human=self.human,
                                          count=self.count)

        if not isinstance(self.actor, Actor): # Need to resolve .actor
            # .actor is currently unresolved name string for actor
            actor, inits, ioinits, parms = self.registrar.__fetch__(self.actor)
            inits.update(self.inits or odict())
            if self.prerefs: # preinits inits dict items
                # each key is share src path, and value is list of src fields
                for src, fields in self.prerefs.get('inits', {}).items():
                    src = self.resolvePath(ipath=src, warn=True) # now a share
                    if not fields:  # default is use existing fields
                        fields = self._prepareSrcFields(src, fields)
                    for field in fields:  # requires src pre inited
                        if field in src:  # only update if src has field
                            inits[field] = src[field]
            if 'name' not in inits:
                inits['name'] = self.actor  # as yet unresolved name string
            inits['store'] = self.frame.store
            inits['act'] = self
            self.actor = actor = actor(**inits) # instantiate and convert

            parms.update(self.parms or odict())
            if self.prerefs: # preinits parms dict items
                # each key is share src path, and value is list of src fields
                for src, fields in self.prerefs.get('parms', {}).items():
                    src = self.resolvePath(ipath=src, warn=True) # now a share
                    if not fields:  # default is use existing fields
                        fields = self._prepareSrcFields(src, fields)
                    for field in fields:  # requires src pre inited
                        if field in src:  # only update if src has field
                            parms[field] = src[field]

            ioinits.update(self.ioinits or odict())
            if self.prerefs: # preinits ioinits dict items
                # each key is share src path, and value is list of src fields
                for src, fields in self.prerefs.get('ioinits', {}).items():
                    src = self.resolvePath(ipath=src, warn=True) # now a share
                    if not fields:  # default is use existing fields
                        fields = self._prepareSrcFields(src, fields)
                    for field in fields:  # requires src pre inited
                        if field in src:  # only update if src has field
                            ioinits[field] = src[field]
            if ioinits:
                iois = actor._initio(**ioinits)
                if iois:
                    for key, ioi in iois.items():
                        share = actor._resolvePath(   ipath=ioi['ipath'],
                                                     ival=ioi.get('ival'),
                                                     iown=ioi.get('iown'))
                        if actor._Parametric:
                            if key in parms:
                                msg = "ResolveError: Parm and Ioi with same name"
                                raise excepting.ResolveError(msg,
                                                             key,
                                                             self,
                                                             self.human,
                                                             self.count)
                            parms[key] = share
                        else:
                            if hasattr(actor, key):
                                msg = "ResolveError: Attribute and Ioi with same name"
                                raise excepting.ResolveError(msg,
                                                             key,
                                                             self,
                                                             self.human,
                                                             self.count)
                            setattr(actor, key, share)
            self.parms = parms
            self.parms.update(self.actor._resolve(**self.parms)) # resolve sub acts
            self.actor._prepare(**self.parms)

    def resolvePath(self, ipath, ival=None, iown=None, warn=False):
        """ Returns resolved Share or Node instance from ipath
            ipath may be path name of share or node
            or reference to Share or Node instance

            This method resolves pathname strings into share and node references
            at resolve time.

            ival is optional value for share
            iown is optional ownership if True then overwrite if exists
                   otherwise do not overwrite

            if warn then will complain if the share is created.

            It allows for substitution into ipath of
            frame, framer, actor, main frame, or main framer relative names.
            So that lexically relative pathnames can
            be dynamically resolved in support of framer cloning.
            It assumes that any implied variants have been reconciled.
            If .actor is not yet instantiated then raises exception if ipath
               uses actor relative addressing. This may occur for init prerefs.

            When ipath is a string:  (not a Node or Share reference)
                the following syntax is used:

                If the path name starts with a leading '.' dot then path name is
                fully reconciled and no contextual substitutions are to be applied.

                Otherwise make subsitutions in pathname strings that begin
                with 'framer.'
                Substitute for special path part 'framer' with names of 'me' or 'main'
                Substitute for special path part 'frame' with names  of 'me' or 'main'
                Substitute for special path part 'actor' with name of 'me'

                'me' indicates substitute the current framer, frame, or actor name
                respectively.

                'main' indicates substitute the current frame's main framer or main frame
                name respectively obtained from

            When  ipath is a pathname string that resolves to a Share and ival is not None
            Then ival is used to initialize the share values.
                ival should be a share initializer:
                   valid initializers are:
                       a dict of fields and values
                       a list of duples, each a (key, value) item

                If own is True then .update(ival) is used
                Otherwise .create(ival) is used

            Requires that:
               self.store exist

               The following have already been resolved:
                   self.frame
                   self.frame.framer
                   self.frame.framer.main
                   self.frame.framer.main.framer

               If actor relative addressing is used then:
                  self.actor is resolved
                  self.actor.name is not empty

        """
        if not (isinstance(ipath, storing.Share) or isinstance(ipath, storing.Node)): # must be pathname
            if not ipath.startswith('.'): # not reconciled so do relative substitutions
                parts = ipath.split('.')
                if parts[0] == 'me':  #  framer inode relative addressing
                    if not self.frame:
                        raise excepting.ResolveError("ResolveError: Missing frame context"
                                " to resolve relative pathname.", ipath, self,
                                self.human, self.count)
                    if not self.frame.framer:
                        raise excepting.ResolveError("ResolveError: Missing framer context"
                                " to resolve relative pathname.", ipath, self.frame,
                                self.human, self.count)
                    del parts[0]

                    finode = ''  # inode of framer
                    framer = self.frame.framer
                    finode = framer.inode
                    if framer.main:  # aux framer so special meaning of main or mine
                        if finode == "main":  # replace finode with main framer inode
                            finode = framer.main.framer.inode
                        elif finode == "mine":  # don't prepend main framer inode
                            finode = ""
                        else:  # possibly prepend main framer inode if finode relative
                            minode = framer.main.framer.inode
                            minode = minode.rstrip('.')
                            if not finode.startswith('.') and minode:  # prepend if relative
                                finode = '.'.join([minode, finode])
                    finode = finode.rstrip('.')
                    if finode:
                        finparts = finode.split('.')
                        if finparts:
                            parts = finparts + parts
                    ipath = '.'.join(parts)

            if not ipath.startswith('.'): # not reconciled so do relative substitutions
                parts = ipath.split('.')
                if parts[0] == 'framer':  #  relative addressing
                    if not self.frame:
                        raise excepting.ResolveError("ResolveError: Missing frame context"
                            " to resolve relative pathname.", ipath, self,
                                self.human, self.count)
                    if not self.frame.framer:
                        raise excepting.ResolveError("ResolveError: Missing framer context"
                            " to resolve relative pathname.", ipath, self.frame,
                                self.human, self.count)

                    if parts[1] == 'me': # current framer
                        parts[1] = self.frame.framer.name
                    elif parts[1] == 'main': # current main framer
                        if not self.frame.framer.main:
                            raise excepting.ResolveError("ResolveError: Missing main framer"
                                " context to resolve relative pathname.", ipath,
                                self.frame.framer,
                                self.human, self.count)
                        parts[1] = self.frame.framer.main.framer.name

                    if (len(parts) >= 3):
                        if parts[2] == 'frame':
                            if parts[3] == 'me':
                                parts[3] = self.frame.name
                            elif parts[3] == 'main':
                                if not self.frame.framer.main:
                                    raise excepting.ResolveError("ResolveError: "
                                        "Missing main frame context to resolve "
                                        "relative pathname.", ipath,
                                        self.frame.framer,
                                        self.human, self.count)
                                parts[3] = self.frame.framer.main.name
                            if (len(parts) >= 5 ) and parts[4] == 'actor':
                                if parts[5] == 'me': # slice insert multiple parts
                                    if (not isinstance(self.actor, Actor)
                                            or not self.actor.name): # unresolved actor
                                        raise excepting.ResolveError("ResolveError: Unresolved actor"
                                            " context to resolve relative pathname.", ipath, self,
                                                        self.human, self.count)
                                    parts[5:6] = nameToPath(self.actor.name).lstrip('.').rstrip('.').split('.')
                        elif parts[2] == 'actor':
                            if parts[3] == 'me': # slice insert multiple parts
                                if (not isinstance(self.actor, Actor)
                                        or not self.actor.name):
                                    raise excepting.ResolveError("ResolveError: Unresolved actor"
                                        " context to resolve relative pathname.", ipath, self,
                                               self.human, self.count)
                                parts[3:4] = nameToPath(self.actor.name).lstrip('.').rstrip('.').split('.')


                ipath = '.'.join(parts)

            if not self.frame.store:
                raise excepting.ResolveError("ResolveError: Missing store context"
                                " to resolve relative pathname.", ipath, self,
                                self.human, self.count)

            if ipath.endswith('.'): # Node not Share
                ipath = self.frame.store.createNode(ipath.rstrip('.'))
                if warn:
                    console.profuse( "     Warning: Non-existent node '{0}' "
                                        "... creating anyway\n".format(ipath))
            else: # Share
                ipath = self.frame.store.create(ipath)
                if ival is not None:
                    if iown:
                        ipath.update(ival)
                    else:
                        ipath.create(ival)
                if warn:
                    console.profuse( "     Warning: Non-existent node '{0}' "
                                     "... creating anyway\n".format(ipath))

        return ipath

    def _prepareSrcFields(self, src, fields):
        """
        Prepares and verifys list field names fields in share src
           handles default conditions when fields is empty

           src is share
           fields is list of field names

        """
        if not fields: # empty source fields so assign defaults
            if src:
                if 'value' in src:
                    fields = ['value'] #use value field
                else: #use pre-existing source fields
                    fields = src.keys()
            else: # empty src
                fields = ['value'] #use value field
        return fields

class Nact(Act):
    """ Negating Act used for actor needs to give Not Need
    """
    __slots__ = ('frame', 'context', 'act',
                 'actor', 'registrar', 'parms', 'inits', 'ioinits',
                 'human', 'count')

    def __init__(self, **kwa ):
        """ Initialization method for instance. """
        super(Nact,self).__init__(**kwa)

    def __call__(self): #make Act instance callable as function
        """Define act as callable object
           Negate the output
        """
        return not (self.actor(**self.parms))

    def expose(self):
        """ Show attributes """
        console.terse("Nact Actor {0} Parms {1} in Frame {2} Context {3} SuperAct {4}\n".format(
                    self.actor, self.parms, self.frame, self.context, self.act))
        if self.actor:
            self.actor._expose()

class SideAct(Act):
    """ Anciliary act to a main Act/Actor used to call a different 'action' method
        of the Main act.actor in support of combined activity such as restarting
        an action.

        SideActs are created in the .resolve method of an Actor and assume that
        all attributes and parms have already been resolved.

        Unique Attributes:
            .action = name of method to call in .actor
    """
    __slots__ = ('action')

    def __init__(self, action='', **kwa ):
        """ Initialization method for instance. """
        super(SideAct, self).__init__(**kwa)
        self.action = action

    def __call__(self): #make Act instance callable as function
        """ Define call method named .action of .actor """
        return (getattr(self.actor, self.action)(**self.parms))

    def resolve(self, **kwa):
        """ Assumes all has been resolved.
            Check for valid action
        """
        if not isinstance(self.actor, Actor): # Woops
            msg = "ResolveError: Unresolved actor"
            raise excepting.ResolveError(msg, self.actor, self)

        if not self.action : # Woops
            msg = "ResolveError: Empty action"
            raise excepting.ResolveError(msg,
                                         self.action,
                                         self,
                                         self.human,
                                         self.count)

        if not getattr(self.actor, self.action, None):
            msg = "ResolveError: Missing action in actor"
            raise excepting.ResolveError(msg,
                                         self.action,
                                         self,
                                         self.human,
                                         self.count)

def actify(name, base=None, registry=None, inits=None, ioinits=None, parms=None,
             parametric=None):
    """ Parametrized decorator function that converts the decorated function
        into an Actor sub class with .action method and with class name that
        is name and registers the new subclass in the registry under name as
        a subclass of base. The default base is Actor.

        The parameters  registry, parametric, inits, ioinits, and parms if provided,
        are used to create the class attributes for the new subclass

    """
    base = base or Actor
    if not issubclass(base, Actor):
        msg = "Base class '{0}' not subclass of Actor".format(base)
        raise excepting.RegisterError(msg)

    attrs = odict()
    if registry:
        attrs['Registry'] = odict()
    if parametric is not None:
        attrs['_Parametric'] = parametric
    if inits:
        attrs['Inits'] = odict(inits)
    if ioinits:
        attrs['Ioinits'] = odict(ioinits)
    if parms:
        attrs['Parms'] = odict(parms)
    cls = type(name, (base, ), attrs )

    def implicit(func):
        @wraps(func)
        def inner(*pa, **kwa):
            return func(*pa, **kwa)
        cls.action = inner
        return inner
    return implicit

actorify = actify  # alias for backwards compatibility

@metaclassify(registering.RegisterType) # python2or3 compatible
class Actor(object):
    """ Actor Base Class
        Has Actor specific Registry of classes
    """
    Registry = odict() # Actor Registry
    Inits = odict() # class defaults support for
    Ioinits = odict() # class defaults
    Parms = odict() # class defaults
    _Parametric = True # Convert iois to action method parameters if Truthy
    __slots__ = ('name', 'store', '_act')

    def __init__(self, name='', store=None, act=None, **kwa ):
        """
        Initialization method for instance.

        Instance attributes
            .name = name string for Actor variant in class Registry
            .store = reference to shared data Store
            ._act = reference to containing Act

        If subclass has init need to call super
        super(SubClassName, self).__init__(**kwa)

        """
        self.name = name
        if store is not None:
            if  not isinstance(store, storing.Store):
                raise ValueError("Not store {0}".format(store))
            self.store = store
        self._act = act


    def __call__(self, **kwa):
        """ run .action  """
        return self.action(**kwa)

    def action(self, **kwa):
        """Action called by Actor. Should override in subclass."""
        console.profuse("Actioning {0} in {1} of {2} with {3}\n".format(self.name,
                                                                        self._act.frame.name,
                                                                        self._act.frame.framer.name,
                                                                        str(kwa)))
        pass

    def _expose(self):
        """Show Actor."""
        console.terse("Actor {0}".format(self.name))

    def _resolve(self, **kwa):
        """ Return updated parms
            Extend in subclass to resolve specific kwa items that are links or
            share refs and update parms
        """
        parms = odict()

        return parms

    def _initio(self, inode='', **kwa):
        """ Intialize and hookup ioflo shares from node pathname inode and kwa arguments.
            This implements a generic Actor interface protocol for associating the
            io data flow shares to the Actor.

            inode is the computed pathname string of the default share node
            where shares associated with the Actor instance may be placed when
            relative addressing is used.

            The values of the items in the **kwa argument may be either strings or
            mappings

            For each key,val in **kwa.items() there are the following 2 forms for val:

            1- string:
               ipath = pathnamestring
            2- dict of items (mapping) of form:
                {
                    ipath: "pathnamestring", (optional)
                    ival: initial value, (optional)
                    iown: truthy, (optional)
                }

            In either case, three init values are produced, these are:
                ipath, ival, iown,
            Missing init values from ipath, ival, iown will be assigned a
            default value as per the rules below:

            For each kwa item (key, val)
                key is the name of the associated instance attribute or action parm.
                Extract ipath, ival, iown from val

                Shares are initialized with mappings passed into share.create or
                share.update. So to assign ival to share.value pass into share.create
                or share.update a mapping of the form {'value': ival} whereas
                passing in an empty mapping does nothing.

                If ipath not provided
                    ipath is the default path inode.key
                Else ipath is provided
                    If ipath starts with dot "." Then absolute path
                    Else ipath does not start with dot "." Then relative path from inode

                    If ipath ends with dot Then the path is to a node not share
                            node ref is created and remaining init values are ignored


                If ival not provided
                     set ival to empty mapping which when passed to share.create
                     will not change share.value
                Else ival provided
                    If ival is an empty Mapping Then
                        assign a shallow copy of ival to share.value by passing
                        in {value: ival (copy)} to share.create/.update
                    Else If ival is a non-string iterable and not a mapping
                        assign a shallow copy of ival to share.value by passing
                        in {value: ival (copy)} to share.create/.update
                    Else If ival is a non-empty Mapping Then
                        Each item in ival is assigned as a field, value pair in the share
                        by passing ival directly into share.create or share.update
                        This means there is no way to init a share.value to a non empty mapping
                        It is possible to init a share.value to an empty mapping see below
                    Else
                        assign ival to share.value by by passing
                        in {value: ival} to share.create or share.update

                Create share with pathname given by ipath

                If iown Then
                    init share with ival value using update
                Else
                    init share with ival value using create ((change if not exist))

        """
        if not isinstance(inode, basestring):
            raise ValueError("Nonstring inode arg '{0}'".format(inode))

        finode = ''  # inode of framer
        if self._act.frame.framer:
            framer = self._act.frame.framer
            finode = framer.inode
            if framer.main:  # aux framer so special meaning of main or mine
                if finode == "main":  # replace finode with main framer inode
                    finode = framer.main.framer.inode
                elif finode == "mine":  # don't prepend main framer inode
                    finode = ""
                else:  # possibly prepend main framer inode if finode relative
                    minode = framer.main.framer.inode
                    minode = minode.rstrip('.')
                    if not finode.startswith('.') and minode:  # prepend if relative
                        finode = '.'.join([minode, finode])

        finode = finode.rstrip('.')

        if not inode.startswith('.') and finode:  # not absolute and finode so prepend
            inode = ".".join([finode, inode])

        if not inode:  # empty or missing use default actor relative
            inode = "framer.me.frame.me.actor.me."

        if not inode.endswith('.'):
            inode = "{0}.".format(inode)

        iois = odict()

        for key, val in kwa.items():
            if val is None:
                continue

            if isinstance(val, basestring):
                ipath = val
                iown = None
                ival = odict() # effectively will not change share
            elif isinstance(val, Mapping): # dictionary
                ipath = val.get('ipath')
                iown = val.get('iown')

                if not 'ival' in val:
                    ival = odict() # effectively will not change share
                else:
                    ival = val['ival']
                    if isinstance(ival, Mapping):
                        if not ival: #empty mapping
                            ival = odict(value=copy.copy(ival)) #make copy so each instance unique
                        # otherwise don't change since ival is non-empty mapping
                    elif nonStringIterable(ival): # not mapping and nonStringIterable
                        ival = odict(value=copy.copy(ival))
                    else:
                        ival = odict(value=ival)
            else:
                raise ValueError("Bad init kw arg '{0}'with Value '{1}'".format(key, val))

            if ipath:
                if not ipath.startswith('.'): # full path is inode joined to ipath
                    ipath = '.'.join((inode.rstrip('.'), ipath)) # when inode empty prepends dot
            else:
                ipath = '.'.join((inode.rstrip('.'), key))  # when ipath empty create default from key

            if ipath.startswith('me.') or ipath == 'me':
                raise ValueError("Invalid ipath '{0}', first part == 'me'".format(ipath))

            ioi = odict(ipath=ipath, ival=ival, iown=iown)
            iois[key] = ioi

        ioi = odict(ipath=inode)
        iois['inode'] = ioi

        return iois # non-empty when _parametric

    def _prepare(self, **kwa):
        """ Base method to be overriden in sub classes. Perform post initio setup
            after all parms and ioi parms or attributes have been created
            kwa usually parms.

            If one is using this method consider refactoring into two different
            behaviors
        """
        pass

    def _resolvePath(self, ipath, ival=None, iown=None, warn=False):
        """ Returns resolved Share or Node instance from ipath
            Calls self._act.resolvePath()
            See doc string from Act.resolvePath for detailed description of
            functionality

            Requires that
               self._act exist
        """
        if not (isinstance(ipath, storing.Share) or isinstance(ipath, storing.Node)): # must be pathname
            if not self._act:
                raise excepting.ResolveError("ResolveError: Missing act context"
                        " to resolve relative pathname.", ipath, self,
                        self._act.human, self._act.count)
            ipath = self._act.resolvePath(ipath, ival, iown, warn)

        return ipath

    def _prepareDstFields(self, srcFields, dst, dstFields):
        """
        Prepares  for a transfer of data
        from srcFields to dstFields in dst
           handles default conditions when fields are empty

        srcFields is list of field names
        dst is share
        dstFields is list of field names

        """

        if not dstFields: #no destinationField so use default rules
            if 'value' in dst:
                dstFields = ['value'] #use value field

            else: #use srcFields
                dstFields = srcFields

        self._verifyShareFields(dst, dstFields)

        if len(srcFields) != len(dstFields):
            msg = ("ResolveError: Unequal number of source = {0} and "
                   "destination = {2} fields".format(srcFields, dstFields))
            raise excepting.ResolveError(msg,
                                         self.name,
                                         '',
                                         self._act.human,
                                         self._act.count)

        for dstField, srcField in izip(dstFields, srcFields):
            if (dstField != srcField) and (srcField != 'value'):
                console.profuse("     Warning: Field names mismatch. '{0}' in {1} "
                                "from '{2}' ... creating anyway".format(
                                  dstField, dst.name, srcField))

        #create any non existent destination fields
        for field in dstFields: #use destination fields for destination data
            if field not in dst:
                console.profuse("     Warning: Transfer into non-existent field '{0}' in "
                       "share {1} ... creating anyway\n".format(field, dst.name))
                dst[field] = None #create

        return dstFields

    def _prepareSrcDstFields(self, src, srcFields, dst, dstFields):
        """
        Prepares and verifys a transfer of data
           from srcFields in src
           to dstFields in dst
           handles default conditions when fields are empty

           src and dst are shares
           srcFields and dstFields are lists

        """
        if not srcFields: # empty source fields so assign defaults
            if src:
                if 'value' in src:
                    srcFields = ['value'] #use value field
                elif dstFields: #use destination fields for source fields
                    srcFields = dstFields
                else: #use pre-existing source fields
                    srcFields = src.keys()
            else: # empty src
                srcFields = ['value'] #use value field

        self._verifyShareFields(src, srcFields)

        if not dstFields: #no destination fields so assign defaults
            if 'value' in dst:
                dstFields = ['value'] #use value field
            else: #use source fields for destination fields
                dstFields = srcFields

        self._verifyShareFields(dst, dstFields)

        if len(srcFields) != len(dstFields):
            msg = ("ResolveError: Unequal number of fields, source = {0} and"
                  " destination={1)".format(srcFields, dstFields))
            raise excepting.ResolveError(msg,
                                         self.name,
                                         '',
                                         self._act.human,
                                         self._act.count)

        for dstField, srcField in izip(dstFields, srcFields):
            if (dstField != srcField) and (srcField != 'value'):
                console.profuse("     Warning: Field names mismatch. '{0}' in {1} "
                                "from '{2}' in {3}  ... creating anyway".format(
                                    dstField, dst.name, srcField, src.name))

        #create any non existent source or destination fields
        for field in srcFields: #use source fields for source data
            if field not in src:
                console.profuse("     Warning: Transfer from non-existent field '{0}' "
                        "in share {1} ... creating anyway".format(field, src.name))
                src[field] = None #create

        for field in dstFields: #use destination fields for destination data
            if field not in dst:
                console.profuse("     Warning: Transfer into non-existent field '{0}' "
                        "in share {1} ... creating anyway\n".format(field, dst.name))
                dst[field] = None #create

        return (srcFields, dstFields)

    def _verifyShareFields(self, share, fields):
        """Verify that updating fields in share won't violate the
           condition that when a share has field == 'value'
           it will be the only field

           fields is list of field names
           share is  share

           raises exception if condition would be violated
        """
        if (len(fields) > 1) and ('value' in fields):
            msg = "ResolveError: Field = 'value' within fields = '{0}'".format(fields)
            raise excepting.ResolveError(msg,
                                         self.name,
                                         '',
                                         self._act.human,
                                         self._act.count)

        if share: #does share have fields already
            for field in fields:
                if field not in share: #so this field could be added to share
                    if ('value' in share) or (field == 'value'):
                        msg = ("ResolveError: Candidate field '{0}' when "
                               "fields = '{1}' exist".format(field, share.keys()))
                        raise excepting.ResolveError(msg,
                                                     self.name,
                                                     '',
                                                     self._act.human,
                                                     self._act.count)

class Interrupter(Actor):
    """Interrupter Actor Class
       Interrupter is a base clase for all actor classes that interrupt normal precur
       processing and result in a change in the frame or the frame processing.

       This class must be subclassed. This is a convenience so can either use
         isinstance to test

       Specifically an Interrupter's action() method returns truthy when its action
       interrupts the normal frame processing.

       Examples are:
       Transiters which interrupt by changing to a new frame
       Suspenders which interrupt when the conditional aux condition is true and
          further processing of the frame and sub frames is stopped


    """
    def __init__(self,**kw ):
        """Initialization method for instance. """
        super(Interrupter,self).__init__(**kw)

class Transiter(Interrupter):
    """Transiter Interrupter Class
       Transiter  is a special actor that performs transitions between frames

       Parameters
            needs = list of Act objects that are exit needs for this trans
            near = source frame of trans
            far = target frame of trans
            human = text version of transition

    """
    def _resolve(self, needs, near, far, human, **kwa):
        """Resolve any links in far and in associated parms for actors"""

        parms = super(Transiter, self)._resolve( **kwa)

        if near == 'me':
            near = self._act.frame

        parms['near'] = near = framing.resolveFrame(near,
                                                    who=self.name,
                                                    desc='near',
                                                    human=self._act.human,
                                                    count=self._act.count)


        if far == 'next':
            if not isinstance(near.next_, framing.Frame):
                raise excepting.ResolveError("ResolveError: Bad next frame",
                                             near.name, near.next_,
                                             self._act.human, self._act.count)
            far = near.next_

        elif far == 'me':
            far = near

        far = framing.resolveFrame(far,
                                   who=self.name,
                                   desc='far',
                                   human=self._act.human,
                                   count=self._act.count)

        parms['far'] = far

        for act in needs:
            act.act = self._act
            act.resolve()

        return parms


    def action(self, needs, near, far, human, **kw):
        """Action called by Actor  """

        framer = near.framer #to speed up

        console.profuse("Attempt segue from {0} to {1}\n".format(near.name, far.name))

        for act in needs:
            if not act(): #return None if not all true
                return None

        console.profuse("     active outline: {0}\n".format([frame.name for frame in framer.actives]))
        console.profuse("     far outline: {0}\n".format([frame.name for frame in far.outline]))

        #find uncommon entry and exit lists associated with transition
        #exits, enters = framing.Framer.Uncommon(framer.actives,far.outline)
        #find uncommon and common entry and exit lists associated with transition
        exits, enters, reexens = framing.Framer.ExEn(framer.actives, far)

        #check enters, if successful, perform transition
        if not framer.checkEnter(enters):
            return None

        msg = "To: {0}<{1} at {2} Via: {3} ({4}) From: {5} after {6:0.3f}\n".format(
            framer.name, far.human, framer.store.stamp, near.name, human,
            framer.human, framer.elapsed)
        console.terse(msg)

        console.profuse("     exits: {0}\n".format([frame.name for frame in exits]))
        console.profuse("     enters: {0}\n".format([frame.name for frame in enters]))
        console.profuse("     reexens: {0}\n".format([frame.name for frame in reexens]))

        framer.exit(exits) #exit uncommon frames in near outline reversed in place
        framer.rexit(reexens[:]) #make copy since reversed in place
        framer.renter(reexens)
        framer.enter(enters)
        framer.activate(active = far)
        return far

    def _expose(self):
        """      """
        console.terse("Transiter {0}\n".format(self.name))


class Suspender(Interrupter):
    """Suspender Interrupter Class
       Suspender  is a special actor that performs a conditional auxiliary
          which truncates the active outline effectively suspended the truncated
          frames

       Parameters
            needs = list of Act objects that are exit needs for this trans
            main = source frame of trans
            aux = target frame of trans
            human = text version of transition

    """
    def __init__(self,**kw ):
        """Initialization method for instance. """
        super(Suspender,self).__init__(**kw)

    def _resolve(self, needs, main, aux, human, **kwa):
        """Resolve any links aux and in associated parms for actors"""
        parms = super(Suspender, self)._resolve( **kwa)

        if main == 'me':
            main = self._act.frame

        parms['main'] = main = framing.resolveFrame(main,
                                                    who=main,
                                                    desc='main',
                                                    human=self._act.human,
                                                    count=self._act.count)

        parms['aux'] = aux = framing.resolveFramer(aux,
                                                   who=main.name,
                                                   desc='aux',
                                                   contexts=[AUX],
                                                   human=self._act.human,
                                                   count=self._act.count)

        for act in needs:
            act.act = self._act
            act.resolve()

        deActParms = odict(aux=aux)
        deAct = SideAct( actor=self,
                        parms=deActParms,
                        action='deactivize',
                        human=self._act.human,
                        count=self._act.count)
        self._act.frame.addExact(deAct)
        console.profuse("{0}Added exact {1} SideAct for {2} with {3} in {4}\n".format(
                INDENT_ADD, 'deactivize', self.name, deAct.parms, self._act.frame.name))
        deAct.resolve()

        return parms

    def action(self, needs, main, aux, human, **kw):
        """Action called by Actor  """

        framer = main.framer #to speed up

        if aux.done: #not active

            console.profuse("Attempt segue from {0} to aux {1}\n".format(main.name, aux.name))

            for act in needs:
                if not act(): #return None if not all true
                    return None

            #if aux.main: #see if aux still belongs to another frame
            #    return None

            # if aux.main is not None then it has not been released and so
            # we can't enter unless it is our act's frame
            if aux.main and (aux.main is not self._act.frame):
                console.concise("    Invalid aux '{0}' in use "
                        "by another frame '{1}'\n".format(aux.name, aux.main.name))
                return None

            if not aux.checkStart(): #performs entry checks
                return None

            msg = "To: {0} in {1}<{2} at {3} Via: {4} ({5}) From: {6} after {7:0.3f}\n".format(
                aux.name, framer.name, main.headHuman, framer.store.stamp, main.name,
                human, framer.human, framer.elapsed)
            console.terse(msg)

            if aux.original:
                aux.main = main  #assign aux's main to this frame
            aux.enterAll() #starts at aux.first frame
            aux.recur()

            if aux.done: #if done after first iteration then clean up
                self.deactivate(aux)
                return None

            #truncate active outline to suspend lower frames
            framer.change(main.head, main.headHuman)
            return aux

        if not aux.done: #not done so active
            aux.segue()
            aux.recur()

            if aux.done: #if done after this iteraion clean up
                self.deactivate(aux)
                framer.reactivate()
                return None

            return aux


    def _expose(self):
        """      """
        console.terse("Suspender {0}\n".format(self.name))

    def deactivize(self, aux, **kwa):
        """ If not aux.done Then force deactivate. Used in exit action."""
        if not aux.done:
            console.profuse("{0} deactivate {1}\n".format(self.name, aux.name))
            self.deactivate(aux)

    def deactivate(self, aux):
        """Called by deactivator actor to cleanly exit      """
        console.profuse("Deactivating {0}\n".format(aux.name))

        aux.exitAll() # also sets .done = True
        if aux.original:
            aux.main = None



class Printer(Actor):
    """Printor Actor Class

       Printer is a special actor that just prints to console its message

    """
    def action(self, message, **kw):
        """Action called by Actor
        """
        #console.terse("{0} printer {1}\n".format(self.name, message))
        console.terse("*** {0} ***\n".format(message))

    def _expose(self):
        """   """
        console.terse("Printer {0}\n".format(self.name))

class Marker(Actor):
    """ Base class that sets up mark in provided share reference"""
    pass

class MarkerUpdate(Marker):
    """ MarkerUpdate Class

        MarkerUpdate is a special actor that acts on a share to mark the update by
            saving a copy of the last stamp

        MarkerUpdate works with NeedUpdate which does the comparison against the marked stamp.

        Builder at parse time when it encounters an NeedUpdate,
        creates the mark in the share and creates the appropriate MarkerUpdate
    """
    def action(self, share, frame, **kwa):
        """ Update mark in share
            Where share is reference to share and frame is frame name key of mark in
                share.marks odict
            Updates mark.stamp

            only one mark per frame per share is needed
        """
        console.profuse("{0} mark {1} in {2} on {3}\n".format(
            self.name, share.name, frame, 'update' ))

        mark = share.marks.get(frame)
        if mark:
            mark.stamp = self.store.stamp #stamp when marker runs

    def _expose(self):
        """   """
        console.terse("MarkerUpdate {0}\n".format(self.name))

class MarkerChange(Marker):
    """ MarkerChange Class

        MarkerChange is a special actor that acts on a share to mark save a copy
        of the data in the mark for the share.

        MarkerChange works with NeedChange which does the comparison with the mark

        Builder at parse time when it encounters a NeedChange,
        creates the mark in the share and creates the appropriate marker
    """
    def action(self, share, frame, **kwa):
        """ Update mark in share
            Where share is reference to share and frame is frame name key of mark in
                share.marks odict
            Updates mark.data

            only one mark per frame per share is needed
        """
        console.profuse("{0} mark {1} in {2} on {3}\n".format(
            self.name, share.name, frame, 'change' ))

        mark = share.marks.get(frame)
        if mark:
            mark.data = storing.Data(share.items())

    def _expose(self):
        """   """
        console.terse("MarkerChange {0}\n".format(self.name))

class Rearer(Actor):
    """
    Rearer Actor Class
    Rearer is a special actor that clones a moot framer at runtime

       Parameters
            original = moot framer to be cloned
            clone = name of clone
            schedule = schedule kind of clone
            frame = frame to put auxiliary clone in

    """
    def _resolve(self, original, clone, schedule, frame, **kwa):
        """
        Resolve any links
        """
        parms = super(Rearer, self)._resolve( **kwa)

        parms['original'] = original = framing.resolveFramer(original,
                                                    who=self.name,
                                                    desc='original',
                                                    contexts=[MOOT],
                                                    human=self._act.human,
                                                    count=self._act.count)

        if schedule not in [AUX]:
            msg = ("ResolveError: Invalid schedule '{0}' for clone"
                  "of '{1}'".format(ScheduleNames.get(schedule, schedule),
                                  original.name))
            raise excepting.ResolveError(msg=msg,
                                         name=self.name,
                                         value=schedule,
                                         human=self._act.human,
                                         count=self._act.count)

        if schedule == AUX:  # only current framer
            framer = framing.resolveFramer(self._act.frame.framer,
                                            who=self._act.frame.name,
                                            desc='rear aux clone',
                                            contexts=[],
                                            human=self._act.human,
                                            count=self._act.count)

            if frame == 'me':  # cannot rear in current frame
                msg = ("ResolveError: Invalid frame 'me' for reared clone.")
                raise excepting.ResolveError(msg,
                                             name=clone,
                                             value=self.name,
                                             human=self._act.human,
                                             count=self._act.count)

            # frame required
            parms['frame'] = frame = framing.resolveFrameOfFramer(frame,
                                                                  framer,
                                                                  who=self._act.frame.name,
                                                                  desc='rear aux clone',
                                                                  human=self._act.human,
                                                                  count=self._act.count)

            if clone != 'mine':
                msg = "ResolveError: Aux insular clone name must be 'mine' not '{0}'".format(clone)
                raise excepting.ResolveError(msg,
                                             name=clone,
                                             value=self.name,
                                             human=self._act.human,
                                             count=self._act.count)

        else:
            msg = ("ResolveError: Invalid insular clone schedule '{0}' "
                                               "for {1}.".format(schedule, original))
            raise excepting.ResolveError(msg,
                                         name=clone,
                                         value=self.name,
                                         human=self._act.human,
                                         count=self._act.count)

        return parms

    def action(self, original, clone, schedule, frame, **kw):
        """Action called by Actor  """

        console.profuse("         Cloning '{0}' as '{1}' be '{2}'\n".format(
                original.name, clone, ScheduleNames.get(schedule, schedule)))

        if schedule == AUX:
            if frame in self._act.frame.outline:
                console.terse("         Error: Cannot rear clone in own"
                              " '{0}' outline. {1} in line {2}\n".format(frame.name,
                                                                         self._act.human,
                                                                         self._act.count))
                return


            clone = framing.Framer.nameUid(prefix=original.name)
            while (clone in framing.Framer.Names): # ensure unique
                clone = framing.Framer.nameUid(prefix=original.name)

            console.terse("         Cloning original '{0}' as aux insular clone"
                          " '{1}' of Frame '{2}'\n".format(original.name,
                                                           clone,
                                                           frame.name))

            clone = original.clone(name=clone, schedule=schedule)
            self._act.frame.framer.assignFrameRegistry()  # restore original.clone changes
            clone.original = False  # main frame will be fixed
            clone.insular = True  #  local to this framer
            clone.razeable = True  # can be razed
            frame.addAux(clone)
            clone.main = frame
            self.store.house.resolvables.append(clone)
            self.store.house.resolveResolvables()


class Razer(Actor):
    """
    Razer Actor Class
    Razer is a special actor that destroys a  framer clone at runtime

       Parameters
            who = auxiliary clones to be destroyed
            frame = frame holding clones to be destroyed

    """
    def _resolve(self, who, frame, **kwa):
        """
        Resolve any links
        """
        parms = super(Razer, self)._resolve( **kwa)


        framer = framing.resolveFramer(self._act.frame.framer,
                                        who=self._act.frame.name,
                                        desc='rear aux clone',
                                        contexts=[],
                                        human=self._act.human,
                                        count=self._act.count)

        if frame == 'me':  # cannot rear in current frame
            frame = self._act.frame

        # frame required
        parms['frame'] = frame = framing.resolveFrameOfFramer(frame,
                                                              framer,
                                                              who=self._act.frame.name,
                                                              desc='rear aux clone',
                                                              human=self._act.human,
                                                              count=self._act.count)

        return parms

    def action(self, who, frame, **kw):
        """Action called by Actor  """

        razeables = []
        if who == 'all':
            for aux in frame.auxes:
                if aux.insular and aux.razeable:
                    razeables.append(aux)

        elif who == 'first':
            for aux in frame.auxes:
                if aux.insular and aux.razeable:
                    razeables.append(aux)
                    break

        elif who == 'last':
            for aux in reversed(frame.auxes):
                if aux.insular and aux.razeable:
                    razeables.append(aux)
                    break

        for aux in razeables:
            console.concise("         Razing '{0}' in '{1}'\n".format(who, frame.name))
            aux.prune()
            frame.auxes.remove(aux)



"""globaling.py module with global constants

"""


import sys
import math
import re

from ..aid.sixing import *  # Python2to3 support

#Globals
#Constant Definitions

#FlowScript build reporting indentations
INDENT_ADD = '      '
INDENT_CREATE = '     '

#Task generator control values
STOP = 0
START = 1
RUN = 2
ABORT = 3
READY =  4
ControlNames = { STOP : 'Stop', START : 'Start', RUN : 'Run', ABORT : 'Abort', READY : 'Ready',}

#Task generator operational status
STOPPED = STOP
STARTED = START
RUNNING = RUN
ABORTED = ABORT
READIED = READY
StatusNames = { STOPPED : 'Stopped', STARTED : 'Started', RUNNING : 'Running', ABORTED : 'Aborted', READIED : 'Readied', }
StatusValues = { 'Stopped' : STOPPED, 'Started' :  STARTED, 'Running' : RUNNING, 'Aborted' : ABORTED, 'Readied' : READIED,}

#log rule values
NEVER = 0
ONCE = 1
ALWAYS = 2
UPDATE = 3
CHANGE = 4
LIFO = 5
FIFO = 6
LogRuleNames = {NEVER : 'Never', ONCE : 'Once', ALWAYS : 'Always',
                UPDATE : 'Update', CHANGE : 'Change', LIFO : 'Lifo', FIFO: 'Fifo',}
LogRuleValues = {'Never' : NEVER, 'Once' : ONCE,  'Always' : ALWAYS,
                 'Update' : UPDATE, 'Change' : CHANGE, 'Lifo' : LIFO, 'Fifo' : FIFO,}

#Task schedule context
INACTIVE = 0
ACTIVE = 1
AUX = 2
SLAVE = 3
MOOT = 4
ScheduleNames = { INACTIVE : 'inactive',
                  ACTIVE : 'active',
                  AUX : 'aux',
                  SLAVE : 'slave',
                  MOOT : 'moot'}
ScheduleValues = { 'inactive': INACTIVE ,
                   'active':  ACTIVE,
                   'aux': AUX,
                   'slave': SLAVE,
                   'moot': MOOT}

#Task ordering
MID = 0
FRONT = 1
BACK = 2

OrderNames = { MID : 'mid', FRONT : 'front', BACK : 'back'}
OrderValues = { 'mid': MID , 'front':  FRONT, 'back': BACK}

#Frame action contexts
NATIVE = 0
ENTER = 1
RECUR = 2
PRECUR = 3
EXIT = 4
RENTER = 5
REXIT = 6
BENTER = 7

ActionContextValues = { 'native': NATIVE , 'enter':  ENTER, 'recur': RECUR,
                        'precur': PRECUR, 'exit': EXIT, 'renter': RENTER, 'rexit': REXIT,
                        'benter': BENTER}
ActionContextNames = { NATIVE : 'native', ENTER : 'enter', RECUR : 'recur',
                       PRECUR : 'precur', EXIT : 'exit', RENTER : 'renter', REXIT : 'rexit',
                       BENTER : 'benter'}


#Precompile re match objects

#regular expression objects to quickly determine if string is valid python identifier
#Usage: REO_Identifier.match('Hello') returns match object if match otherwise None
REO_Identifier = re.compile(r'^[a-zA-Z_]\w*$') #valid python identifier
REO_IdentPub = re.compile(r'^[a-zA-Z]\w*$') #valid python public identifier ie no leading underscore

# regex objects to determine if string is valid store path
# to use
#if REO_Path.match(s):
#  then s is valid path

REO_RelPath = re.compile(r'^([a-zA-Z_]\w*)+([.][a-zA-Z_]\w*)*$')  # not start with dot
REO_DotPath = re.compile(r'^([.][a-zA-Z_]\w*)+$')  # starts with dot
REO_Path = re.compile(r'^([a-zA-Z_]\w*)+([.][a-zA-Z_]\w*)*$|^([.][a-zA-Z_]\w*)+$')  # dotpath or relpath
REO_PathDotPath = re.compile(r'^([a-zA-Z_]\w*)+([.][a-zA-Z_]\w*)+$|^([.][a-zA-Z_]\w*)+$') # at least one dot

# allows path to end in dot or not for node paths
REO_RelPathNode = re.compile(r'^([a-zA-Z_]\w*)+(([.][a-zA-Z_]\w*)*$|([.][a-zA-Z_]\w*)*[.]$)')  # not start with dot
REO_DotPathNode = re.compile(r'^([.][a-zA-Z_]\w*)+$|^([.][a-zA-Z_]\w*)+[.]$')  # starts with dot
REO_PathNode = re.compile(r'^([a-zA-Z_]\w*)+(([.][a-zA-Z_]\w*)*$|([.][a-zA-Z_]\w*)*[.]$)|^([.][a-zA-Z_]\w*)+$|^([.][a-zA-Z_]\w*)+[.]$')

#regex object to split hafscript command line
#REO_Chunks = re.compile(r'#.*|[^ "]+|"[^"]*"')
REO_Chunks = re.compile(r"""#.*|[^ "']+|"[^"]*"|'[^']*'""")
# Usage
# chunks = REO_Chunks.findall(s)

# to match each part
REO_Quoted = re.compile(r'^"[^"]*"$')
REO_QuotedSingle = re.compile(r"^'[^']*'$")
REO_Comment = re.compile(r'^#.*$')
REO_Plain = re.compile(r'^[^ "]+$')
#Usage
# if REO_Quoted.match(s):
#    s.strip('"')

#regex object determine if lat or lon is in human readable form
#REO_LATLONPOS = re.compile(r'^([0-9]+)[N,E,n,e]([0-9]+\.[0-9]+)$')
REO_LatLonNE = re.compile(r'^(\d+)[N,E,n,e](\d+\.\d+)$')
REO_LatLonSW = re.compile(r'^(\d+)[S,W,s,w](\d+\.\d+)$')
#Usage
# ll = REO_LatLonNE.findall(s) #returns list of tuples of groups [(deg,min)]
# if ll:
#   deg = float(ll[0][0])
#   min = float(ll[0][1])
#   fracdeg = deg + min/60.0





# -*- coding: utf-8 -*-
"""
Unit Test Template
"""
from __future__ import absolute_import, division, print_function

import sys
if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

import os

from ioflo.aid.sixing import *
from ioflo.test import testing
from ioflo.base.consoling import getConsole
console = getConsole()

from ioflo.aid import blending


def setUpModule():
    console.reinit(verbosity=console.Wordage.concise)

def tearDownModule():
    pass


class BasicTestCase(unittest.TestCase):
    """
    Example TestCase
    """

    def setUp(self):
        """
        Call super if override so House Framer and Frame are setup correctly
        """
        super(BasicTestCase, self).setUp()

    def tearDown(self):
        """
        Call super if override so House Framer and Frame are torn down correctly
        """
        super(BasicTestCase, self).tearDown()

    def testBlending(self):
        """
        Test the blend functions
        """
        console.terse("{0}\n".format(self.testBlending.__doc__))
        u = .25
        s = .75
        steps = 10
        u = abs(u)
        s = abs(s)
        steps = abs(steps)
        span = u + s
        ss = span / steps
        out = []
        for x in xrange(-(steps + 1), steps + 2, 1):
            d = x * ss
            b = blending.blend0(d,u,s)
            out.append((round(d, 3), round(b, 3)))

        self.assertEqual(out, [(-1.1, 0.0),
                                (-1.0, 0.0),
                                (-0.9, 0.133),
                                (-0.8, 0.267),
                                (-0.7, 0.4),
                                (-0.6, 0.533),
                                (-0.5, 0.667),
                                (-0.4, 0.8),
                                (-0.3, 0.933),
                                (-0.2, 1.0),
                                (-0.1, 1.0),
                                (0.0, 1.0),
                                (0.1, 1.0),
                                (0.2, 1.0),
                                (0.3, 0.933),
                                (0.4, 0.8),
                                (0.5, 0.667),
                                (0.6, 0.533),
                                (0.7, 0.4),
                                (0.8, 0.267),
                                (0.9, 0.133),
                                (1.0, 0.0),
                                (1.1, 0.0)])

        out = []
        for x in xrange(-(steps + 1), steps + 2, 1):
            d = x * ss
            b = blending.blend1(d,u,s)
            out.append((round(d, 3), round(b, 3)))

        self.assertEqual(out, [(-1.1, 1.0),
                                (-1.0, 1.0),
                                (-0.9, 1.0),
                                (-0.8, 1.0),
                                (-0.7, 1.0),
                                (-0.6, 1.0),
                                (-0.5, 1.0),
                                (-0.4, 1.0),
                                (-0.3, 1.0),
                                (-0.2, 1.0),
                                (-0.1, 0.564),
                                (0.0, 0.0),
                                (0.1, 0.564),
                                (0.2, 1.0),
                                (0.3, 1.0),
                                (0.4, 1.0),
                                (0.5, 1.0),
                                (0.6, 1.0),
                                (0.7, 1.0),
                                (0.8, 1.0),
                                (0.9, 1.0),
                                (1.0, 1.0),
                                (1.1, 1.0)])

        out = []
        for x in xrange(-(steps + 1), steps + 2, 1):
            d = x * ss
            b = blending.blend2(d,u,s)
            out.append((round(d, 3), round(b, 3)))

        self.assertEqual(out, [(-1.1, 1.0),
                                (-1.0, 1.0),
                                (-0.9, 1.0),
                                (-0.8, 1.0),
                                (-0.7, 0.997),
                                (-0.6, 0.987),
                                (-0.5, 0.95),
                                (-0.4, 0.853),
                                (-0.3, 0.66),
                                (-0.2, 0.381),
                                (-0.1, 0.113),
                                (0.0, 0.0),
                                (0.1, 0.113),
                                (0.2, 0.381),
                                (0.3, 0.66),
                                (0.4, 0.853),
                                (0.5, 0.95),
                                (0.6, 0.987),
                                (0.7, 0.997),
                                (0.8, 1.0),
                                (0.9, 1.0),
                                (1.0, 1.0),
                                (1.1, 1.0)])

        out = []
        for x in xrange(-(steps + 1), steps + 2, 1):
            d = x * ss
            b = blending.blend3(d,u,s)
            out.append((round(d, 3), round(b, 3)))

        self.assertEqual(out, [(-1.1, 0.996),
                                (-1.0, 0.99),
                                (-0.9, 0.976),
                                (-0.8, 0.947),
                                (-0.7, 0.895),
                                (-0.6, 0.809),
                                (-0.5, 0.684),
                                (-0.4, 0.521),
                                (-0.3, 0.339),
                                (-0.2, 0.168),
                                (-0.1, 0.045),
                                (0.0, 0.0),
                                (0.1, 0.045),
                                (0.2, 0.168),
                                (0.3, 0.339),
                                (0.4, 0.521),
                                (0.5, 0.684),
                                (0.6, 0.809),
                                (0.7, 0.895),
                                (0.8, 0.947),
                                (0.9, 0.976),
                                (1.0, 0.99),
                                (1.1, 0.996)])

def runOne(test):
    '''
    Unittest Runner
    '''
    test = BasicTestCase(test)
    suite = unittest.TestSuite([test])
    unittest.TextTestRunner(verbosity=2).run(suite)

def runSome():
    """ Unittest runner """
    tests =  []
    names = ['testBlending',
            ]
    tests.extend(map(BasicTestCase, names))
    suite = unittest.TestSuite(tests)
    unittest.TextTestRunner(verbosity=2).run(suite)

def runAll():
    """ Unittest runner """
    suite = unittest.TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(BasicTestCase))
    unittest.TextTestRunner(verbosity=2).run(suite)

if __name__ == '__main__' and __package__ is None:

    #console.reinit(verbosity=console.Wordage.concise)

    #runAll() #run all unittests

    runSome()#only run some

    #runOne('testBasic')





"""detecting.py detector deed module

"""
#print("module {0}".format(__name__))

import math
import time
import struct
from collections import deque
import inspect

from ....aid.sixing import *
from ....aid.odicting import odict
from ....base.globaling import *

from ....aid import aiding, navigating
from ....base import doing

from ....base.consoling import getConsole
console = getConsole()

class DetectorPositionBox(doing.Doer):
    """
    Detects if vehicle position is in box or out
    output share indicates which side in or out
    """
    Ioinits = odict(
        group = 'detector.position.box',
        output = 'box', input = 'state.position',
        parms = dict(track = 0.0, north = 0.0, east = 0.0,
                     length = 10000, width = 2000, turn = 45.0))

    def __init__(self, **kw):
        """Initialize instance.
           group is path name of group in store, group has following subgroups or shares:
           local outputs


           inherited instance attributes
           .name
           .store

        """
        #call super class method
        super(DetectorPositionBox,self).__init__(**kw)


    def _initio(self, group, output, input, parms = None, **kw):
        """ Override since uses legacy interface

            output
              inside   = vehicle is inside box (Booleans True (False))
              outtop  = vehicle is outside box above top highest priority
              outbottom = vehicle is outside box below bottom next priority
              outleft  = vehicle is outside to the left
              outright  = vehicle is outside to the right
              turnleft = heading for left turn
              turnright = heading for right turn

           inputs
           input = share path name to input vehicle position (north, east)

           parms = dictionary to initialize group.parm fields
              parm.track  = azimuth of track in (degrees)
              parm.width  = width of box  (meters)
              parm.length = length of box (meters)
              parm.north  = north value for center bottom of box (meters)
              parm.east  = east value for center bottom of box (meters)
              parm.turn = degrees to left or right of track for zig zags (0 - 90)

           instance attributes

           .group = copy of group name

           .output = ref to output
              side state with boolean data fields
              .inside, .outtop, .outbottom, .outleft, .outright
              turn headings with data fields
              .turnleft
              .turnright

           .input = ref to vehicle position external input

           .parm = ref to input parameter share group.parm

        """
        self.group = group

        #local outputs
        self.output = self.store.create(output)#create if not exist
        #order important for display purposes
        fields = odict(inside = False)
        fields['outtop'] = False
        fields['outbottom'] = False
        fields['outleft'] = False
        fields['outright'] = False
        fields['turnleft'] = 0.0
        fields['turnright'] = 0.0
        self.output.update(fields)

        #inputs position
        self.input = self.store.create(input)
        self.input.create(north = 0.0)
        self.input.create(east = 0.0)

        #parms
        self.parm = self.store.create(group + '.parm') #create if not exist
        if not parms:
            parms = dict(track = 0.0, width = 1000, length = 10000,
                         north = 0.0, east = 0.0, turn = 45.0)

        parms['turn'] = abs(navigating.Wrap2(parms['turn']))
        self.parm.create(**parms)

        turnleft = self.parm.data.track - self.parm.data.turn
        turnright = self.parm.data.track + self.parm.data.turn

        self.output.update(turnleft = turnleft, turnright = turnright)


    def action(self, **kw):
        """computes box detector output

           treat center bottom as origin
           inside outside as delta position relative to center bottom
           and rotated about center bottom by -track
        """
        #compute turns
        turnleft = navigating.Wrap2(self.parm.data.track - self.parm.data.turn)
        turnright = navigating.Wrap2(self.parm.data.track + self.parm.data.turn)

        self.output.update(turnleft = turnleft, turnright = turnright)

        #get input vehicle position
        pn = self.input.data.north
        pe = self.input.data.east

        #get parameters
        cbn = self.parm.data.north
        cbe = self.parm.data.east
        track = self.parm.data.track
        length = self.parm.data.length
        width = self.parm.data.width

        #get offset of vehicle position relative to center bottom
        pn = pn - cbn
        pe = pe - cbe
        #rotate vehicle position to box by - track
        pn, pe = navigating.RotateFSToNE(heading = -track, forward = pn, starboard = pe)

        #compute side tests
        self.output.update(inside = True)  #default inside until shown outside

        #topright to topleft
        cn = length #top left corner
        ce = - width/2.0
        sn = 0.0 #top side from top left to top right
        se = width
        rn = pn - cn #vehicle position relative to top left corner
        re = pe - ce
        #print(cn,ce,sn,se,pn,pe,rn,re)
        side = se * rn - sn * re #2d perp product
        if side >= 0.0:
            self.output.update(outtop = True, inside = False)
        else:
            self.output.update(outtop = False)

        #bottomright to bottomleft
        cn = 0.0 #bottom right corner
        ce = width/2.0
        sn = 0.0 #bottom side from bottom right to bottom left
        se = - width
        rn = pn - cn #vehicle position relative to top left corner
        re = pe - ce
        side = se * rn - sn * re #2d perp product
        if side >= 0.0:
            self.output.update(outbottom = True, inside = False)
        else:
            self.output.update(outbottom = False)

        #bottomleft to topleft
        cn = 0.0  #bottom left corner
        ce = - width/2.0
        sn = length #left side from bottom left to top left
        se = 0.0
        rn = pn - cn #vehicle position relative to top left corner
        re = pe - ce
        side = se * rn - sn * re #2d perp product
        if side >= 0.0:
            self.output.update(outleft = True, inside = False)
        else:
            self.output.update(outleft = False)

        #topright to bottomright
        cn = length  #top right corner
        ce = width/2.0
        sn = - length #right side from top right to bottom right
        se = 0.0
        rn = pn - cn #vehicle position relative to top left corner
        re = pe - ce
        side = se * rn - sn * re #2d perp product
        if side >= 0.0:
            self.output.update(outright = True, inside = False)
        else:
            self.output.update(outright = False)

        #print("cn = %0.3f ce = %0.3f sn = %0.3f se = %0.3f rn = %0.3f re = %0.3f side = %0.3f" %\
        #   (cn,ce,sn,se,rn,re,side))

        if console._verbosity >= console.Wordage.profuse:
            self._expose()
            console.profuse("doing {0} with output (itblf) = {1}\n".format(self.name, self.output))

    def _expose(self):
        """
           prints out detector state

        """
        print("Detector %s" % (self.name))
        format = "box center bottom north = %0.3f east = %0.3f"
        print(format % (self.parm.data.north, self.parm.data.east))
        format = "box track = %0.3f length = %0.3f width = %0.3f turn = %0.3f"
        print(format % (self.parm.data.track, self.parm.data.length,
                        self.parm.data.width, self.parm.data.turn))
        format = "turn left = %0.3f turn right  = %0.3f "
        print(format % (self.output.data.turnleft, self.output.data.turnright))
        format = "position north = %0.3f east = %0.3f"
        print(format % (self.input.data.north, self.input.data.east))
        format = "box inside = %s outside top = %s bottom = %s left = %s right = %s"
        print(format % (self.output.data.inside,
                        self.output.data.outtop, self.output.data.outbottom,
                        self.output.data.outleft, self.output.data.outright))





""" ioflo package

"""
from __future__ import division

import importlib

_modules = ['base', 'trim']  # register behaviors

for m in _modules:
    importlib.import_module(".{0}".format(m), package='ioflo')

from .__metadata__ import *




"""skedding.py weightless thread scheduling


"""
#print( "module {0}".format(__name__))

import sys
if sys.version > '3':
    xrange = range
import os
import time
from collections import deque

from .consoling import getConsole
console = getConsole()

from ..aid.sixing import *
from ..aid.odicting import odict
from .globaling import *

from ..aid import timing
from . import excepting
from . import registering
from . import storing
from . import tasking
from . import building

from ..__metadata__ import __version__

from .consoling import getConsole
console = getConsole()


class Skedder(object):
    """Schedules weightless tasker objects based on generators.

       run method runs the skedder main loop until interrupted or all taskers
       completed

       taskers is a dictionary of taskers indexed by tasker name

       The skedder maintains lists of taskers in various execution states
          Each list determines what the skedder does with the tasker.
          The skedder has methods that move taskers between the lists and
              also notifies taskers of their control

          Skedder runs tasker and sends it a control
          Tasker runs using control and yields its status

          Each tasker as a .desire attribute that indicates what the next
          desired control should be.

          Each tasker as a .period attribute that indicates how ofter the tasker
          should be run

          There are three deques the skedder maintains. Each entry in each deque
          is a tuple (tasker, retime, period)
             tasker is reference to tasker object
             retime is time that the tasker should next be run
                a retime of zero means runs asap or always
             period is the time period between runs

          ready = deque of tuples where taskers are ready to be run
             If need different priorities then need to add a
                ready list for each priority

          stopped = deque of tuples where taskers stopped awaiting start

          aborted = deque of tuples where taskers aborted can't be restarted

          addStoppedTask(tasker) adds tasker to stopped list
          addReadyTask(tasker) adds tasker to ready list

       Everytime a tasker runs it yields a status that the skedder uses to determine
       what to do with the tasker

       instance attributes:
       .name = skedder name string
       .period = time seconds between iterations of skedder
       .stamp = current iteration time of skedder
       .real = real time IF True ELSE simulated time
       .timer = timer to time loops in real time
       .elapsed = timer to time elapsed in mission

       .houses = list of houses to be scheduled

       .ready = deque of tasker  tuples ready to run
       .aborted = deque of tasker tuples aborted
    """

    def __init__(  self,
                   name="skedder",
                   period=0.125,
                   stamp=0.0,
                   real=False,
                   retro=True,
                   filepath='',
                   behaviors=None,
                   username='',
                   password='',
                   mode=None,
                   houses=None,
                   metas=None,
                   preloads=None, ):
        """
        Initialize Skedder instance.
        parameters:
            name = name string
            period = iteration period
            stamp = initial time stamp value
            real = time mode real time True or simulated time False
            retro = shift timers if retrograded system clock detected
            filepath = filepath to build file
            behaviors = list of pathnames to packages with external behavior modules
            username = username
            password = password
            mode = parsing mode
            houses = list of houses
            metas = list of triples of (name, path, data) where
                name = name string of house attribute, path = path string, data = odict
            preloads = list of duples of (path, data) to preload Store where
               path = path string, data = odict
        """
        self.name = name
        self.period = float(abs(period))

        self.stamp = float(abs(stamp))
        #real time or sim time mode
        self.real = True if real else False
        self.timer = timing.MonoTimer(duration = self.period, retro=retro)
        self.elapsed = timing.MonoTimer(retro=retro)

        self.filepath = os.path.abspath(filepath)
        self.plan = os.path.split(self.filepath)[1]
        self.behaviors = behaviors or []
        self.username = username
        self.password = password
        self.mode = mode or []
        self.houses = houses or []

        #Meta data format is list of triples of form (name, path, value)
        self.metas = [
                ("name", "meta.name", odict(value=self.name)),
                ("period", "meta.period", odict(value=self.period)),
                ("real", "meta.real", odict(value=self.real)),
                ("mode", "meta.mode", odict(value=self.mode)), #applied mode logging only
                ("plan", "meta.plan", odict(value=self.plan)),
                ("filepath", "meta.filepath", odict(value=self.filepath)),
                ("behaviors", "meta.behaviors", odict(value=self.behaviors)),
                ("credentials", "meta.credentials",
                     odict([('username', self.username), ('password', self.password)])),
                ("failure", "meta.failure", odict(value="")), # for failure reporting
                ("framers", "meta.framers", odict()), # for failure reporting
                ("taskables", "meta.taskables", odict(value=set())), # to add taskables at runtime
            ]
        if metas:
            self.metas.extend(metas)

        self.preloads = [
                ("ioflo.version", odict(value=__version__)),
                ("ioflo.platform",
                     odict([("os", sys.platform),
                            ("python", "{0}.{1}.{2}".format(*sys.version_info)),] )),

            ]
        if preloads:
            self.preloads.extend(preloads)

        self.ready = deque() # deque of taskers in run order
        self.aborted = deque() # deque of aborted taskers

    def addReadyTask(self, tasker):
        """
        Prepare tasker to be started and add to ready list
        """
        if tasker.schedule == ACTIVE:
            tasker.desire = START
        else:
            tasker.desire = STOP
        tasker.status = STOPPED
        retime = tasker.store.stamp
        period = tasker.period
        trp = (tasker, retime, period)
        self.ready.append(trp)
        console.profuse("     Add ready: {0} retime: {1} period: {2} desire {3}\n".format(
            tasker.name, retime, period, ControlNames[tasker.desire]))

    def build(self, filepath='', mode=None, metas=None, preloads=None):
        """ Build houses from file given by filepath """

        console.terse("Building Houses for Skedder '{0}' ...\n".format(self.name))
        #use parameter otherwise use inited value
        if filepath:
            self.filepath = filepath
        if mode:
            self.mode.extend(mode)
        if metas:
            self.metas.extend(metas)
        if preloads:
            self.preloads.extend(preloads)

        b = building.Builder(fileName = self.filepath,
                             mode=self.mode,
                             metas = self.metas,
                             preloads =self.preloads,
                             behaviors=self.behaviors)

        if not b.build():
            return False

        self.houses = b.houses

        for house in self.houses:
            console.profuse("Meta Data for House '{0}':\n{1}\n".format(
                house.name, house.metas))

        return True

    def run(self, growable=False):
        """runs all generator taskers in running list by calling next() method.

           Keyboard interrupt (cntl-c) to end forever loop
           Since finally clause closes taskers they must be restarted before
           run can be executed again

           if growable is True then allow adding new taskers at runtime
              via  house metas['taskables']

        """

        console.terse("Starting Skedder '{0}' ...\n".format(self.name))

        stamp = self.stamp
        for house in self.houses:
            house.store.changeStamp(stamp)
            ("Initialized store {0}:  stamp = {1} with {2}\n".format(
                house.store.name,  house.store.stamp, stamp))

            for tasker in house.taskables:
                self.addReadyTask(tasker)

        console.profuse("Ready Taskers: {0}\n".format(
            ', '.join([tasker.name for tasker,r,p in self.ready])))
        console.profuse("Aborted Taskers: {0}\n".format(
            ', '.join([tasker.name for tasker,r,p in self.aborted])))


        self.timer.restart()
        self.elapsed.restart()

        #make local reference for speed put out side loop?
        ready = self.ready
        #stopped = self.stopped
        aborted = self.aborted

        try: #so always clean up resources if exception
            while True:
                try: #CNTL-C generates keyboardInterrupt to break out of while loop
                    console.profuse("\nRunning Skedder '{0}' at stamp = {1} real elapsed = {2:0.4f}\n".format(
                        self.name, self.stamp,  self.elapsed.elapsed))

                    more = False #are any taskers RUNNING or STARTED

                    for i in xrange(len(ready)): #attempt to run each ready tasker
                        tasker, retime, period = ready.popleft() #pop it off

                        if retime > stamp: #not time yet
                            ready.append((tasker, retime, period)) #reappend it
                            status = tasker.status

                        else: #run it
                            try:
                                status = tasker.runner.send(tasker.desire)
                                if status == ABORTED: #aborted so abort tasker
                                    aborted.append((tasker, stamp, period))
                                    console.profuse("     Tasker Self Aborted: {0}\n".format(tasker.name))
                                else:
                                    ready.append((tasker,
                                                  retime + tasker.period,
                                                  tasker.period))  # append allows for period change

                            except StopIteration: #generator returned instead of yielded
                                aborted.append((tasker, stamp, period))
                                console.profuse("     Tasker Aborted due to StopIteration: {0}\n".format(tasker.name))

                        if status == RUNNING or status == STARTED:
                            more = True

                    if growable:
                        # todo from each house.metas fetch new taskables
                        # add to ready
                        pass

                    if not ready: #no pending taskers so done
                        console.terse("No ready taskers. Shutting down skedder ...\n")
                        break

                    if not more: #all taskers stopped or aborted
                        console.terse("No running or started taskers. Shutting down skedder ...\n")
                        break

                    #update time stamps
                    if self.real:
                        console.profuse("     Time remaining skedder = {0:0.4f}\n".format(self.timer.remaining))
                        while not self.timer.expired:
                            time.sleep(self.timer.remaining)
                        self.timer.repeat()

                    self.stamp += self.period
                    stamp = self.stamp
                    for house in self.houses:
                        house.store.changeStamp(stamp)

                except KeyboardInterrupt: #CNTL-C shutdown skedder
                    console.terse("KeyboardInterrupt forcing shutdown of Skedder ...\n")
                    break

                except SystemExit: #User know why shutting down
                    console.terse("SystemExit forcing shutdown of Skedder ...\n")
                    raise

                except Exception: #Let user know what exception caused shutdoen
                    console.terse("Surprise exception forcing shutdown of Skedder ...\n")
                    raise

            console.terse("Total elapsed real time = {0:0.4f}\n".format(self.elapsed.elapsed))

        finally: #finally clause always runs regardless of exception or not
            #Abort any running taskers to reclaim resources
            #Stopped or aborted taskers should have already released resources
            #if last run tasker exited due to exception then try finally clause in
            #its generator is responsible for releasing resources

            console.terse("Aborting all ready Taskers ...\n")
            for i in xrange(len(ready)): #run each ready tasker once
                tasker,retime,period = ready.popleft() #pop it off

                try:
                    status = tasker.runner.send(ABORT)
                    console.terse("Tasker '{0}' aborted\n".format(tasker.name))
                except StopIteration: #generator returned instead of yielded
                    console.terse("Tasker '{0}' generator already exited\n".format(tasker.name))

                #tasker.runner.close() #kill generator

        if console._verbosity >= console.Wordage.concise:
            for house in self.houses:
                #show store hierarchy
                console.concise( "\nData Store for {0}\n".format(house.name))
                house.store.expose(values=(console._verbosity >= console.Wordage.terse))


def Test(real = False, verbose = False):
    """Module Common self test

    """
    import housing
    reload(housing)

    housing.ClearRegistries()

    print(housing.Registries)
    print("")
    print(housing.Registries["tasker"].Names)
    print(housing.Registries["tasker"].Counter)
    print("")

    house = housing.House()

    t1 = tasking.Tasker(name = 't1', store = house.store)
    t2 = tasking.Tasker(name = 't2', store = house.store)
    t3 = tasking.Tasker(name = 't3', store = house.store, period = 0.125)
    t4 = tasking.Tasker(name = 't4', store = house.store, period = 0.125)
    t5 = tasking.Tasker(name = 't5', store = house.store, period = 0.5)
    t6 = tasking.Tasker(name = 't6', store = house.store, period = 1.0)

    house.actives = [t1,t6,t2,t5,t3,t4]

    skedder = Skedder(name = "TestTasker", period = 0.125, real = real, houses = [house])
    skedder.run()


def TestProfile(real = False, verbose = False):
    """Module Common self test



    """
    import cProfile
    import pstats

    import housing
    reload(housing)

    housing.ClearRegistries()

    print(housing.Registries)
    print("")
    print(housing.Registries["tasker"].Names)
    print(housing.Registries["tasker"].Counter)
    print("")

    house = housing.House()

    t1 = Tasker(name = 't1', store = house.store)
    t2 = Tasker(name = 't2', store = house.store)
    t3 = Tasker(name = 't3', store = house.store, period = 0.125)
    t4 = Tasker(name = 't4', store = house.store, period = 0.125)
    t5 = Tasker(name = 't5', store = house.store, period = 0.5)
    t6 = Tasker(name = 't6', store = house.store, period = 1.0)

    house.actives = [t1,t6,t2,t5,t3,t4]

    skedder = Skedder(name = "TestSkedder", period = 0.125, real = real, houses = [house])
    #skedder.run()
    cProfile.runctx('skedder.run()',globals(),locals(), './test/profiles/skeddertest')

    p = pstats.Stats('./test/profiles/skeddertest')
    p.sort_stats('time').print_stats()
    p.print_callers()
    p.print_callees()

if __name__ == "__main__":
    Test()




""" base package


"""
#print("Package at {0}".format(__path__[0]))
import importlib

_modules = ['globaling', 'excepting', 'interfacing',
           'registering', 'storing', 'skedding',
           'tasking', 'framing', 'logging', 'serving', 'monitoring',
           'acting', 'poking', 'goaling', 'needing', 'traiting',
           'fiating', 'wanting','completing','doing', 'deeding', 'arbiting',
           'housing', 'building']


for m in _modules:
    importlib.import_module(".{0}".format(m), package='ioflo.base')




"""aiding.py constants and basic functions

"""
from __future__ import absolute_import, division, print_function

import sys
import math


# Import ioflo libs
from .sixing import *
from .odicting import odict
from ..base import excepting

from ..base.consoling import getConsole
console = getConsole()

TWOPI = 2.0 * math.pi  # two times pi
DEGTORAD = math.pi / 180.0  # r = DEGTORAD * d
RADTODEG = 180.0 / math.pi  # d = RADTODEG * r

def Sign(x):
    """Calculates the sign of a number and returns
       1 if positive
       -1 if negative
       0 if zero
       should make it so type int or float of x is preserved in return type
    """
    if x > 0.0:
        return 1.0
    elif x < 0.0:
        return -1.0
    else:
        return 0.0


def Delta(desired, actual, wrap = 180.0):
    """Calculate the short rotation for delta = desired - actual
       and delta wraps around at wrap

    """
    #delta = desired  - actual  so
    #desired  = actual  + delta

    return Wrap2(angle = (desired - actual), wrap = wrap)


def Wrap2(angle, wrap = 180.0):
    """Wrap2 = (2 sided one positive one negative) wrap of angle to
       signed interval [-wrap, + wrap] wrap is half circle
       if wrap = 0 then don't wrap
       angle may be positive or negative
       result is invariant to sign of wrap

       Wrap preserves convention so angle can be in compass or Cartesian coordinates

       Uses property of python modulo operator that implement true
       clock or circular arithmetic as location on circle
       distance % circumference = location
       if circumference positive then locations postive sign,
             magnitues increase CW  (CW 0 1 2 3 ... 0)
       if circumference negative then locations negative sign,
             magnitudes increase CCW  (CCW 0 -1 -2 -3 ... 0)

       if distance positive then wrap distance CW around circle
       if distance negative then wrap distance CCW around circle

       No need for a full wrap in Python since modulo operator does that
        even for negative angles
       angle %= 360.0

    """

    if wrap != 0.0:
        angle %= wrap * 2.0 #wrap to full circle first
        if abs(angle) > abs(wrap): #more than half way round
            angle = (angle - wrap) % (- wrap) #wrap extra on reversed half circle

    return angle


def MoveByHSD(heading = 0.0, speed = 1.0, duration = 0.0):
    """
       Returns change in position after moving on heading at speed for duration
       heading in compass coordinates, 0 deg is north, up, cw rotation increases
    """
    deltaNorth = duration * (speed * math.cos(DEGTORAD * heading))
    deltaEast = duration * (speed * math.sin(DEGTORAD * heading))

    return (deltaNorth, deltaEast)

def MoveToHSD(north = 0.0, east = 0.0,
              heading = 0.0, speed = 1.0, duration = 0.0):
    """
       Returns new position after moving on heading at speed for duration
       heading in compass coordinates, 0 deg is north, up, cw rotation increases

       north east order since lat long
    """
    north += duration * (speed * math.cos(DEGTORAD * heading))
    east += duration * (speed * math.sin(DEGTORAD * heading))


    return (north,east)


def RotateFSToNE(heading = 0.0, forward = 0.0, starboard = 0.0):
    """
       rotates Forward Starboard vector to North East vector
       heading in compass coordinates, 0 deg is north, up, cw rotation increases

       north east order since lat long
    """
    ch = math.cos(DEGTORAD * heading)
    sh = math.sin(DEGTORAD * heading)
    north = ch * forward - sh * starboard
    east = sh * forward + ch * starboard

    return (north,east)

def RotateNEToFS(heading = 0.0, north = 0.0, east = 0.0):
    """
       Rotate north east vector to Forward Starboard
       heading in compass coordinates, 0 deg is north, up, cw rotation increases

       north east order since lat long
    """
    ch = math.cos(DEGTORAD * heading)
    sh = math.sin(DEGTORAD * heading)
    forward = ch * north + sh * east
    starboard = - sh * north + ch * east


    return (forward,starboard)


def AlongCrossTrack(track = 0.0, north = 0.0, east = 0.0,
                    mag = None, heading = None):
    """
       Returns as a tuple, the along and cross track components  of the vector
       given by (north, east) where the track is from origin to (n, e)
       or by mag (magnitude) heading (degrees) if provided

       track is the track course ( nav angle degrees)
       a positive along track is in the foreward direction of the track
       a negative along track is in the backward direction of the track
       a positive cross track is to the east of the track
       a negative cross track is to the west of the track
    """
    if mag is not None and heading is not None:
        heading = Wrap2(heading)
        north = mag * math.cos(DEGTORAD * heading)
        east = mag * math.sin(DEGTORAD * heading)

    track = Wrap2(track)

    #along track component
    trackNorth = math.cos(DEGTORAD * track)
    trackEast = math.sin(DEGTORAD * track)

    A = north * trackNorth + east * trackEast

    #cross track vector
    crossNorth = north - A * trackNorth
    crossEast = east - A * trackEast

    #cross track magnitude
    C = (crossNorth ** 2.0 + crossEast ** 2.0) ** 0.5

    #fix sign by testing for shortest rotation of cross vector to track direction
    #if z component of cross X track is positive then shortest rotation is CCW
    # and cross is to the right of track
    #if z component of cross x track is negative then shortest rotation is CW
    # and cross is to the left of track

    (x,y,z) = CrossProduct3D((crossEast, crossNorth, 0.0),
                             (trackEast, trackNorth,0.0))

    if z < 0.0: #make C negative if to left of track
        C *= -1

    return (A,C)

def CrabSpeed(track = 0.0,  speed = 2.0, north = 0.0, east = 0.0,
              mag = None, heading = None):
    """
       Returns a tuple of the compensating (crabbed) course angle (in degrees)
       and the delta crab angle
       and the resulting along track speed (including current and cluster).
       The crabbed course is that needed to compensate for the current
       given by (east, north) or mag (magnitude) heading (degrees) if provided
       Where the resulting along track speed is the projection of
       the compensating course at speed onto the desired course

       track is the desired track course ( nav angle degrees)
       speed is the cluster speed (must be non zero)

       compensating course = desired course - delta crab angle
       a positive crab angle means the compensating course is to the left
       of the desired course.
       a negative crab angle means the compensating course is to the right
       of the desired course
    """
    if mag is not None and heading is not None:
        heading = Wrap2(heading)
        north = mag * math.cos(DEGTORAD * heading)
        east = mag * math.sin(DEGTORAD * heading)

    track = Wrap2(track)
    (A,C) = AlongCrossTrack(track = track, north = north, east = east)
    #current compensated course crab = track + delta crab angle
    delta = - RADTODEG * math.asin(C / speed)
    crab = track + delta
    #B = along track component of compensated course
    B = speed * (math.sin(DEGTORAD * crab) * math.sin(DEGTORAD * track) +
                 math.cos(DEGTORAD * crab) * math.cos(DEGTORAD * track)  )
    return (crab, delta, B + A)

def CrossProduct3D(a,b):
    """Forms the 3 dimentional vector cross product of sequences a and b
       a is crossed onto b
       cartesian coordinates
       returns a 3 tuple
    """
    cx = a[1] * b[2] - b[1] * a[2]
    cy = a[2] * b[0] - b[2] * a[0]
    cz = a[0] * b[1] - b[0] * a[1]

    return (cx,cy,cz)

def DotProduct(a,b):
    """Returns the N dimensional vector dot product of sequences a and b

    """
    dot = 0.0
    for i in range(len(a)):
        dot += a[i] * b[i]

    return dot

def PerpProduct2D(a,b):
    """Computes the the 2D perpendicular product of sequences a and b.
       The convention is a perp b.
       The product is:
          positive if b is to the left of a
          negative if b is to the right of a
          zero if b is colinear with a
       left right defined as shortest angle (< 180)
    """
    return (a[0] * b[1] - a[1] * b[0])

def DistancePointToTrack2D(a,track, b):
    """Computes the signed distance between point b and  the track ray defined by
       point a and track azimuth track
       a and b are sequences x (east) coord is  index 0, y (north) coord is index 1
       track in degrees from north
       x = east
       y = north

       The distance is
          positive if b is to the left of the track line
          negative if b is to the right of the track line
          zero if b is colinear with the track line
       left right defined as shortest angle (< 180)
    """
    dx = math.sin(DEGTORAD * track) #normalized vector
    dy = math.cos(DEGTORAD * track) #normalized vector

    return (dx * (b[1] - a[1]) - dy * (b[0] - a[0]))

def SpheroidLLLLToDNDE(a,b):
    """Computes the flat earth approx of change in north east position meters
       for a change in lat lon location on spheroid.
       from location lat0 lon0 to location lat1 lon1
       point lat0 lon0  in total fractional degrees north east positive
       point lat1, lon1 in total fractional degrees north east positive
       returns tuple (dn,de) where dn is delta north and de is delta east meters
       Uses WGS84 spheroid
    """
    re = 6378137.0 #equitorial radius in meters
    f = 1/298.257223563 #flattening
    e2 = f*(2.0 - f) #eccentricity squared


def SphereLLLLToDNDE(lat0,lon0,lat1,lon1):
    """Computes the flat earth approx of change in north east position meters
       for a change in lat lon location on sphere.
       from location lat0 lon0 to location lat1 lon1
       point lat0 lon0  in total fractional degrees north east positive
       point lat1, lon1 in total fractional degrees north east positive
       returns tuple (dn,de) where dn is delta north and de is delta east meters
       Uses sphere 1 nm = 1 minute 1852 meters per nautical mile
    """
    r = 6366710.0 #radius of earth in meters = 1852 * 60 * 180/pi

    dlat = (lat1 - lat0)
    dlon = (lon1 - lon0)

    avlat = (lat1 + lat0)/2.0
    #avlat = lat0

    dn = r * dlat * DEGTORAD
    de = r * dlon * DEGTORAD * math.cos( DEGTORAD * avlat)

    return (dn, de)

def SphereLLByDNDEToLL(lat0,lon0,dn,de):
    """Computes new lat lon location on sphere
       from the flat earth approx of  change in position dn (north) meters
       and de (east) meters from the given location lat0 lon0
       point lat0 lon0  in total fractional degrees north east positive
       returns tuple (lat1,lon1)
       Uses sphere 1 nm = 1 minute 1852 meters per nautical mile
    """
    r = 6366710.0 #radius of earth in meters = 1852 * 60 * 180/pi

    dlat = dn/(r * DEGTORAD)
    lat1 = lat0 + dlat
    avlat = (lat1 + lat0)/2.0

    try:
        dlon = de / (r * DEGTORAD * math.cos(DEGTORAD * avlat))
    except ZeroDivisionError:
        dlon = 0.0

    lon1 = lon0 + dlon

    avlat = (lat1 + lat0)/2.0

    return (lat1, lon1)

def SphereLLbyRBtoLL(lat0,lon0,range,bearing):
    """Computes new lat lon location on sphere
        from the flat earth approx of  change in range meters at bearing degrees from
         from the given location lat0 lon0
       point lat0 lon0  in total fractional degrees north east positive
       returns tuple (lat1,lon1)
       Uses sphere 1 nm = 1 minute 1852 meters per nautical mile
    """
    r = 6366710.0 #radius of earth in meters = 1852 * 60 * 180/pi

    dn = range * math.cos(DEGTORAD * bearing)
    de = range * math.sin(DEGTORAD * bearing)

    dlat = dn/(r * DEGTORAD)
    lat1 = lat0 + dlat
    avlat = (lat1 + lat0)/2.0

    try:
        dlon = de / (r * DEGTORAD * math.cos(DEGTORAD * avlat))
    except ZeroDivisionError:
        dlon = 0.0

    lon1 = lon0 + dlon

    avlat = (lat1 + lat0)/2.0

    return (lat1, lon1)

def SphereLLLLToRB(lat0,lon0,lat1,lon1):
    """Computes the flat earth approx of change in range meters bearing degrees
       for a change in lat lon location.
       from location lat0 lon0 to location lat1 lon1
       point lat0 lon0  in total fractional degrees north east positive
       point lat1, lon1 in total fractional degrees north east positive
       returns tuple (dn,de) where dn is delta north and de is delta east meters
       Uses sphere 1 nm = 1 minute 1852 meters per nautical mile
    """
    r = 6366710.0 #radius of earth in meters = 1852 * 60 * 180/pi

    dlat = (lat1 - lat0)
    dlon = (lon1 - lon0)

    avlat = (lat1 + lat0)/2.0
    #avlat = lat0

    dn = r * dlat * DEGTORAD
    de = r * dlon * DEGTORAD * math.cos( DEGTORAD * avlat)

    range = (dn * dn + de * de) ** 0.5
    bearing = RADTODEG * ((math.pi / 2.0) - math.atan2(dn,de))

    return (range, bearing)


def RBToDNDE(range, bearing):
    """Computes change in north east position for an offset
       of range (meters) at bearing (degrees)
       returns tuple(delta north, delta East)
    """
    dn = range * math.cos(DEGTORAD * bearing)
    de = range * math.sin(DEGTORAD * bearing)

    return (dn, de)

def DNDEToRB(dn ,de):
    """Computes relative range (meters) and bearing (degrees)for change
       in position of north (meters) east (meters)
       returns tuple(Range, Bearing)
    """
    range = (dn * dn + de * de) ** 0.5
    bearing = RADTODEG * ((math.pi / 2.0) - math.atan2(dn,de))

    return (range, bearing)

def DegMinToFracDeg(latDeg, latMin, lonDeg, lonMin):
    """Converts location in separated format of Deg and Min
       to combined format of total fractional degrees
       lat is in signed fractional degrees positive = North negative = South
       lon in in signed fractional dregrees positive = East negative = West
       latDeg are in signed degrees North positive South Negative
       latMin are in signed minutes North positive South Negative
       lonDeg are in signed degrees East positive West Negative
       lonMin are in signed minutes East positive West Negative
    """
    if Sign(latDeg) != Sign(latMin):
        latMin = - latMin

    if Sign(lonDeg) != Sign(lonMin):
        lonMin =  - lonMin

    lat = latDeg + (latMin / 60.0)
    lon = lonDeg + (lonMin / 60.0)
    return (lat, lon)

def FracDegToDegMin(lat, lon):
    """Converts location in format of total fractional degrees to
       separated format of deg and minutes
       lat is in signed fractional degrees positive = North negative = South
       lon in in signed fractional dregrees positive = East negative = West
       latDeg are in signed degrees North positive South Negative
       latMin are in signed minutes North positive South Negative
       lonDeg are in signed degrees East positive West Negative
       lonMin are in signed minutes East positive West Negative
    """
    latDeg = int(lat)
    latMin = (lat - latDeg) * 60.0

    lonDeg = int(lon)
    lonMin = (lon - lonDeg) * 60.0

    return (latDeg, latMin, lonDeg, lonMin)

def FracDegToHuman(lat, lon):
    """Converts location in format of total fractional degrees to
       tuple (latDM, lonDM) of human friendly string of form
       latDegXlatMin where X is N if lat positive and S if lat negative
          latDeg in units of integer degrees [0 ,90]
          lat Min in units of fractinal minutes [0.0, 60.0)
       and
       lonDegXlonMin where X is E if lon positive and W if lon negative
          lonDeg in units of integer degrees [0 ,180]
          lon Min in units of fractinal minutes [0.0, 60.0)

       lat is in signed fractional degrees positive = North, negative = South
          [-90, 90]
       lon in in signed fractional dregrees positive = East, negative = West
          [-180, 180]

       Does not handle wrapping lat over poles or lon past halfway round
    """
    latDeg, latMin, lonDeg, lonMin = FracDegToDegMin(lat, lon)

    if latDeg >= 0:
        latDM = "%dN%0.3f" % (latDeg, latMin)
    else:
        latDM = "%dS%0.3f" % (-latDeg, -latMin)

    if lonDeg >= 0:
        lonDM = "%dE%0.3f" % (lonDeg, lonMin)
    else:
        lonDM = "%dW%0.3f" % (-lonDeg, -lonMin)

    return (latDM, lonDM)

def HumanLatToFracDeg(latDM):
    """Converts latDM  in human friendly string of form
       latDegXlatMin where X is N if lat positive and S if lat negative
          latDeg in units of integer degrees [0 ,90]
          lat Min in units of fractinal minutes [0.0, 60.0)

       to lat in total fractional degrees

       lat is in signed fractional degrees positive = North, negative = South
          [-90, 90]

       Does not handle wrapping lat over poles or lon past halfway round
    """
    latDM = latDM.upper()
    if ('N' in latDM):
        (degrees,minutes) = latDM.split('N')
        lat = int(degrees) + (float(minutes) / 60.0)

    elif ('S' in latDM):
        (degrees,minutes) = latDM.split('S')
        lat = - (int(degrees) + (float(minutes) / 60.0))

    else:
        raise ValueError("Bad format for latitude '{0}'".format(latDM))

    return (lat)

def HumanLonToFracDeg(lonDM):
    """Converts  lonDM  in human friendly string of form
       lonDegXlonMin where X is E if lon positive and W if lon negative
          lonDeg in units of integer degrees [0 ,180]
          lon Min in units of fractinal minutes [0.0, 60.0)

       to lon in total fractional degrees

       lon in in signed fractional dregrees positive = East, negative = West
          [-180, 180]

       Does not handle wrapping lat over poles or lon past halfway round
    """
    lonDM = lonDM.upper()
    if ('E' in lonDM):
        (degrees,minutes) = lonDM.split('E')
        lon = int(degrees) + (float(minutes) / 60.0)

    elif ('W' in lonDM):
        (degrees,minutes) = lonDM.split('W')
        lon = - (int(degrees) + (float(minutes) / 60.0))

    else:
        raise ValueError("Bad format for longitude '{0}'".format(lonDM))

    return (lon)

def HumanToFracDeg(latDM, lonDM):
    """Converts  pair of coordinates  in human friendly strings of form   DegXMin to
       total fractional degrees where
       the result is positive if X is N or E and
       the result is negative if X is S or W

       Does not handle wrapping over poles or past halfway round
    """
    lat = HumanLatToFracDeg(latDM)
    lon = HumanLonToFracDeg(lonDM)
    return (lat,lon)

def HumanLLToFracDeg(hdm):
    """Converts  a coordinate  in human friendly string of form   DegXMin to
       total fractional degrees where
       the result is positive if X is N or E and
       the result is negative if X is S or W

       Does not handle wrapping over poles or past halfway round
    """
    dm = REO_LatLonNE.findall(hdm) #returns list of tuples of groups [(deg,min)]
    if dm:
        deg = float(dm[0][0])
        min_ = float(dm[0][1])
        return (deg + min_/60.0)

    dm = REO_LatLonSW.findall(hdm) #returns list of tuples of groups [(deg,min)]
    if dm:
        deg = float(dm[0][0])
        min_ = float(dm[0][1])
        return (-(deg + min_/60.0))

    raise ValueError("Bad format for lat or lon '{0}'".format(hdm))

def Midpoint(latDM0, lonDM0, latDM1, lonDM1):
    """Computes the midpoint  of a trackline between
       (latDM0,lonDM0) and (latDM1,lonDM1)
       arguments are in human friendly degrees fractional minutes format
       40N35.67  70W56.45
    """
    lat0 = HumanLLToFracDeg(latDM0)
    lon0 = HumanLLToFracDeg(lonDM0)
    lat1 = HumanLLToFracDeg(latDM1)
    lon1 = HumanLLToFracDeg(lonDM1)

    dn, de = SphereLLLLToDNDE(lat0,lon0,lat1,lon1)
    dn = dn/2.0 #get half the distance
    de = de/2.0
    lat1, lon1 =  SphereLLByDNDEToLL(lat0,lon0,dn,de) #midpoint
    latDM, lonDM = FracDegToHuman(lat1, lon1)

    return (latDM, lonDM)

def Endpoint(latDM0, lonDM0, range, bearing):
    """Computes the endpoint  track from latDM, lonDm of range at bearing

       arguments are in human friendly degrees fractional minutes format
       40N35.67  70W56.45
    """
    lat0 = HumanLLToFracDeg(latDM0)
    lon0 = HumanLLToFracDeg(lonDM0)


    lat1, lon1 = SphereLLbyRBtoLL(lat0,lon0,range,bearing)
    latDM1, lonDM1 = FracDegToHuman(lat1, lon1)

    return (latDM1, lonDM1)



# -*- coding: utf-8 -*-
"""
Unit Test Template
"""
from __future__ import absolute_import, division, print_function

import sys
import datetime
if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

import os

from ioflo.aid.sixing import *
from ioflo.aid.odicting import odict
from ioflo.test import testing
from ioflo.base.consoling import getConsole
console = getConsole()

from ioflo.aid import timing


def setUpModule():
    console.reinit(verbosity=console.Wordage.concise)

def tearDownModule():
    pass


class BasicTestCase(unittest.TestCase):
    """
    Example TestCase
    """

    def setUp(self):
        """
        Call super if override so House Framer and Frame are setup correctly
        """
        super(BasicTestCase, self).setUp()

    def tearDown(self):
        """
        Call super if override so House Framer and Frame are torn down correctly
        """
        super(BasicTestCase, self).tearDown()

    def testTuuid(self):
        """
        Test TUUID generation
        """
        console.terse("{0}\n".format(self.testTuuid.__doc__))
        console.reinit(verbosity=console.Wordage.profuse)

        tuid = timing.tuuid()
        self.assertEqual(len(tuid), 24)
        stamp, sep, randomized = tuid.rpartition('_')
        self.assertEqual(sep, '_')
        self.assertEqual(len(stamp), 16)
        self.assertEqual(len(randomized), 7)

        tuid =  timing.tuuid(9)
        self.assertEqual(len(tuid), 24)
        stamp, sep, randomized = tuid.rpartition('_')
        self.assertEqual(sep, '_')
        self.assertEqual(len(stamp), 16)
        self.assertEqual(len(randomized), 7)
        self.assertEqual(stamp[:16], '0000000000895440' )

        console.reinit(verbosity=console.Wordage.concise)


def runOne(test):
    '''
    Unittest Runner
    '''
    test = BasicTestCase(test)
    suite = unittest.TestSuite([test])
    unittest.TextTestRunner(verbosity=2).run(suite)

def runSome():
    """ Unittest runner """
    tests =  []
    names = [
             'testTuuid',
            ]
    tests.extend(map(BasicTestCase, names))
    suite = unittest.TestSuite(tests)
    unittest.TextTestRunner(verbosity=2).run(suite)

def runAll():
    """ Unittest runner """
    suite = unittest.TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(BasicTestCase))
    unittest.TextTestRunner(verbosity=2).run(suite)

if __name__ == '__main__' and __package__ is None:

    #console.reinit(verbosity=console.Wordage.concise)

    #runAll() #run all unittests

    runSome()#only run some

    #runOne('testBasic')





# -*- coding: utf-8 -*-
"""
Unit Test Template
"""
from __future__ import absolute_import, division, print_function

import sys
if sys.version_info < (2, 7):
    import unittest2 as unittest
else:
    import unittest

import os

from ioflo.aid.sixing import *
from ioflo.test import testing
from ioflo.base.consoling import getConsole
console = getConsole()

from ioflo.aid import odicting


def setUpModule():
    console.reinit(verbosity=console.Wordage.concise)

def tearDownModule():
    pass


class BasicTestCase(unittest.TestCase):
    """
    Example TestCase
    """

    def setUp(self):
        """
        Call super if override so House Framer and Frame are setup correctly
        """
        super(BasicTestCase, self).setUp()

    def tearDown(self):
        """
        Call super if override so House Framer and Frame are torn down correctly
        """
        super(BasicTestCase, self).tearDown()

    def testMoDict(self):
        """
        Test the modict
        """
        console.terse("{0}\n".format(self.testMoDict.__doc__))

        x = odicting.modict()
        x["a"] = 5
        self.assertEqual(x.items(), [('a', 5)])
        x["a"] = 6
        self.assertEqual(x.items(), [('a', 6)])
        self.assertEqual(x.listitems(), [('a', [5, 6])] )

        y = x.setdefault("a", 10)
        self.assertEqual(y, 6)
        z = x.setdefault("b", 1)
        self.assertEqual(z, 1)
        self.assertEqual(x.listitems(), [('a', [5, 6]), ('b', [1])])




def runOne(test):
    '''
    Unittest Runner
    '''
    test = BasicTestCase(test)
    suite = unittest.TestSuite([test])
    unittest.TextTestRunner(verbosity=2).run(suite)

def runSome():
    """ Unittest runner """
    tests =  []
    names = ['testMoDict',
            ]
    tests.extend(map(BasicTestCase, names))
    suite = unittest.TestSuite(tests)
    unittest.TextTestRunner(verbosity=2).run(suite)

def runAll():
    """ Unittest runner """
    suite = unittest.TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(BasicTestCase))
    unittest.TextTestRunner(verbosity=2).run(suite)

if __name__ == '__main__' and __package__ is None:

    #console.reinit(verbosity=console.Wordage.concise)

    #runAll() #run all unittests

    runSome()#only run some

    #runOne('testBasic')





def TestFrame():
    """Module Common self test


    """
    import acting
    import poking
    import needing
    import goaling
    import doing
    import traiting
    import fiating
    import wanting


    try:
        Frame.Clear() #clear registry

        #set up main framework
        f1 = Frame(name = 'Primero')
        f2 = Frame()
        f3 = Frame()
        f1.attach(f2)
        f1.attach(f3)
        f4 = Frame()
        f5 = Frame()
        #f2.attach(Frame())
        f2.attach(f4)
        f3.attach(f5)


        #Frame.ShowHierarchy()

        #Act class Transact class
        Act = acting.Act
        Transact = acting.Transact

        #Action instances
        need = acting.need
        goal = acting.goal
        deed = acting.deed
        trait = acting.trait
        spec = acting.spec
        fiat = acting.fiat

        #a() #t()
        a = Act(action = need, act = need.checkDepth, parms = dict(depth = 5.0))
        f2.beacts.append(a)

        a = Act(action = goal, act = goal.setDepth, parms = dict(depth = 2.0))
        f2.enacts.append(a)

        a = Act(action = trait, act = trait.useDepth, parms = dict(depth = 3.0))
        f2.reacts.append(a)
        a = Act(action = deed, act = deed.doDepth, parms = dict(depth = 1.0))
        f2.reacts.append(a)

        a = Act(action = trait, act = trait.useDepth, parms = dict(depth = 6.0))
        f2.exacts.append(a)

        t = Transact()
        a = Act(action = need, act = need.checkDepth, parms = dict(depth = 4.0))
        t.needs.append(a)
        t.far = f5
        f2.preacts.append(t)

        a = Act(action = deed, act = deed.doDepth, parms = dict(depth = 1.0))
        f2.preacts.append(a)

        t = Transact()
        a = Act(action = need, act = need.checkDepth, parms = dict(depth = 1.5))
        t.needs.append(a)
        t.far = f4
        f5.preacts.append(t)

        #setup auxiliary framework
        f6 = Frame()
        a = Act(action = trait, act = trait.useDepth, parms = dict(depth = 10.0))
        f6.reacts.append(a)
        #setup aux framer
        fr1 = Framer()
        fr1.first = f6
        f3.auxes.append(fr1)

        #a = Act(action = fiat, act = fiat.tellStart, parms = dict(framer = fr1))


        #set up main framer
        fr2 = Framer()
        fr2.first = f1

        fr2.runner.send(START)

        for i in xrange(3):
            status = fr2.runner.send(RUN)


    except excepting.ParameterError as ex:
        console.terse(ex)
        raise

    return f1

def Test():
    """Module Common self test



    """
    TestFrame()


if __name__ == "__main__":
    Test()



"""tasking.py weightless thread scheduling


"""
#print("module {0}".format(__name__))

from ..aid.sixing import *
from .globaling import *
from ..aid.odicting import odict
from . import excepting
from . import registering
from . import storing

from .consoling import getConsole
console = getConsole()


def CreateInstances(store):
    """Create server instances which automatically get registered on object creation
       must be function so can recreate after clear registry
    """
    tasker = Tasker(name = 'tasker', store = store)

#Class definitions


class Tasker(registering.StoriedRegistrar):
    """Task class, Base class for weightless threads

    """
    Counter = 0
    Names = {}

    def __init__(self, period = 0.0, schedule=INACTIVE, **kw):
        """Initialize instance.

           iherited instance attributes
           .name = unique name for tasker
           .store = data store

           instance attributes
           .period = desired time in seconds between runs,non negative, zero means asap
           .stamp = depends on subclass default is time tasker last RUN
           .status = operational status of tasker
           .desire = desired control asked by this or other taskers
           .done = tasker completion state True or False
           .schedule = initial scheduling context for this tasker vis a vis skedder
           .runner = generator to run tasker

            The registry class will supply unique name when name is empty by using
            the .__class__.__name__ as the default preface to the name.
            To use a different default preface add this to the .__init__ method
            before the super call

            if 'preface' not in kw:
                kw['preface'] = 'MyDefaultPreface'

        """
        super(Tasker,self).__init__(**kw)

        self.period = float(abs(period)) #desired time between runs, 0.0 means asap
        self.stamp = 0.0 #time last run
        self.resolved = False  # tasker has been resolved
        self.status = STOPPED #operational status of tasker
        self.desire = STOP #desired control next time Task is iterated
        self.done = True # tasker completion state reset on restart
        self.schedule = schedule #initial scheduling context vis a vis skedder
        self.runner = None #reference to runner generator
        self.remake() #make generator assign to .runner and advance to yield

    def reinit(self, period=None, schedule=None, **kw):
        if period is not None:
            self.period = period

        if schedule is not None:
            self.schedule = schedule

    def remake(self):
        """Re make runner generator

           .send(None) same as .next()
        """
        self.runner = self.makeRunner() #make generator
        status = self.runner.send(None) #advance to first yield to allow send(cmd) on next iteration

        if console._verbosity >= console.Wordage.profuse:
            self.expose()

    def expose(self):
        """

        """
        print("     Task %s status = %s" % (self.name, StatusNames[self.status]))

    def resolve(self, **kwa):
        """Resolves any by name links to other objects   """
        self.resolved = True

    def ready(self):
        """ready runner

        """
        return self.runner.send(READY)

    def start(self):
        """start runner

        """
        return self.runner.send(START)

    def run(self):
        """run runner

        """
        return self.runner.send(RUN)

    def stop(self):
        """stop runner

        """
        return self.runner.send(STOP)

    def abort(self):
        """abort runner

        """
        return self.runner.send(ABORT)

    def makeRunner(self):
        """generator factory function to create generator to run this tasker

           Should be overridden in sub class
        """
        #do any on creation initialization here
        console.profuse("     Making Task Runner {0}\n".format(self.name))

        self.status = STOPPED #operational status of tasker
        self.desire = STOP #default what to do next time, override below
        self.done = True

        count = 0

        try:
            while (True):
                control = (yield (self.status)) #accept control and yield status
                console.profuse("\n     Iterate Tasker {0} with control = {1} status = {2}\n".format(
                    self.name,
                    ControlNames.get(control, 'Unknown'),
                    StatusNames.get(self.status, 'Unknown')))

                if control == RUN:
                    if self.status == STARTED or self.status == RUNNING:
                        console.profuse("     Running Tasker {0} ...\n".format(self.name))
                        self.status = RUNNING
                    else:
                        console.profuse("     Need to Start Tasker {0}\n".format(self.name))
                        self.desire = START

                elif control == READY:
                    console.profuse("     Readying Tasker {0} ...\n".format(self.name))
                    self.desire = START
                    self.status = READIED

                elif control == START:
                    console.terse("     Starting Tasker {0} ...\n".format(self.name))
                    self.desire = RUN
                    self.status = STARTED
                    self.done = False

                elif control == STOP:
                    if self.status == RUNNING or self.status == STARTED:
                        console.terse("     Stopping Tasker {0} ...\n".format(self.name))
                        self.desire = STOP
                        self.status = STOPPED
                        self.done = True
                    else:
                        console.terse("     Tasker {0} not started or running.\n".format(self.name))

                elif control == ABORT:
                    console.profuse("     Aborting Tasker {0} ...\n".format(self.name))
                    self.desire = ABORT
                    self.status = ABORTED
                    self.done = True #only done if complete successfully

                else: #control == unknown error condition bad control
                    self.desire = ABORT
                    self.status = ABORTED
                    console.profuse("     Aborting Tasker {0}, bad control = {1}\n".format(
                        self.name,  CommandNames[control]))
                    break #break out of while loop. this will cause stopIteration

                self.stamp = self.store.stamp

        finally: #in case uncaught exception
            console.profuse("     Exception causing Abort Tasker {0} ...\n".format(self.name))
            self.desire = ABORT
            self.status = ABORTED


def resolveTasker(tasker, who='', desc='tasker', contexts=None, human='', count=None):
    """ Returns resolved tasker instance from tasker
        tasker may be name of tasker or instance
        who is optional name of object owning the link
        such as framer or frame or actor
        desc is string description of tasker link such as 'aux' or 'framer'
        contexts is list of allowed schedule contexts, None or empty means any.
        Taskers.Names registry must already be setup
    """
    if not isinstance(tasker, Tasker): # not instance so name
        if tasker not in Tasker.Names:
            raise excepting.ResolveError("ResolveError: Bad {0} link name".format(desc),
                                         tasker,
                                         who,
                                         human,
                                         count)
        tasker = Tasker.Names[tasker]
        if contexts and tasker.schedule not in contexts:
            raise excepting.ResolveError("ResolveError: Bad {0} link not scheduled"
                                         " as one of {1}".format(desc, contexts),
                                         tasker.name,
                                         who,
                                         human,
                                         count)
        console.concise("         Resolved {0} Tasker '{1}' in '{2}'\n"
                      "".format(desc, tasker.name, who))
    return tasker

ResolveTasker = resolveTasker



"""registering.py support functions and classes

"""
#print("module {0}".format(__name__))

import random

from ..aid.sixing import *
from .globaling import *
from ..aid.odicting import odict

from . import excepting
from ..aid.aiding import reverseCamel

from .consoling import getConsole
console = getConsole()

class RegisterType(type):
    """ Metaclass that registers all subclasses """
    def __init__(cls, name, bases, attrs):
        """ Create if needed and register cls into Registry under reverse of name """
        super(RegisterType, cls).__init__(name, bases, attrs)
        if not hasattr(cls, 'Registry'):
            cls.Registry = odict()
        cls.__register__(   name,
                            inits=getattr(cls, 'Inits', None),
                            ioinits=getattr(cls, 'Ioinits', None),
                            parms=getattr(cls, 'Parms', None))

    def __register__(cls, name, inits=None,  ioinits=None, parms=None):
        """ Register class cls under name with ioinits and inits

            Usage:  A.__register__(name, ioinits, inits, parms)
        """
        if name in cls.Registry:
            msg = "Entry '{0}' already exists in registry of {1}".format(name, cls)
            raise excepting.RegisterError(msg)
        cls.Registry[name] = (cls, inits, ioinits, parms)
        console.profuse("Registered: '{0}'\n".format(name))
        return cls

    def __fetch__(cls, name):
        """ Return tuple derived from .Registry entry at rname if present
            Otherwise raise exception
            Makes copies of inits and ioinits so safe for downstream use
        """
        if name not in cls.Registry:
            msg = "Entry '{0}' not found in Registry of '{1}'".format(name, cls.__name__)
            raise excepting.RegisterError(msg)
        actor, inits, ioinits, parms = cls.Registry[name] # actor is the registered class
        return (actor,
                odict(inits or odict()),
                odict(ioinits or odict()),
                odict(parms or odict()))

class Registrar(object):
    """Class that ensures every instance has a unique name
       uses class variable Counter and  Names dictionary
    """
    __slots__ = ('name', )

    #for base class to have distinct name space shadow these by defining in sub class def
    Counter = 0
    Names = {}

    def __init__(self, name = '', preface = '', **kw):
        """Initializer method for instance.

           instance attributes
           .name = unique name for instance

        """
        self.__class__.Counter += 1 #increment class Counter variable

        if not preface:
            preface = self.__class__.__name__ #use class name of instance as preface

        if not isinstance(name,str): #name must be string
            raise excepting.ParameterError("Expected str instance", "name", name)

        if not name: #if name empty then provide name
            name = str(preface) + str(self.__class__.Counter)
            while (name in self.__class__.Names):
                name += chr(ord('a') + random.randint(0,25)) #add a random letter
        elif (name in self.__class__.Names): #if provided name must be unique
            raise excepting.ParameterError("Instance name attribute not unique", "name", name)

        self.name = name
        self.__class__.Names[self.name] = self #add instance to Names dict indexed by name


    @classmethod
    def Clear(cls):
        """clears (empties) registry of Names and resets Counter to 0

        """
        cls.Names = {}
        cls.Counter = 0

    @classmethod
    def VerifyName(cls, name = ''):
        """verifies that name would be unique non empty string for instance name

           return False if empty or if already in Names , True otherwise
        """
        if not isinstance(name,str): #name must be string
            return False

        if (not name) or (name in cls.Names): #name must be unique and not empty
            return False

        return True

    @classmethod
    def Retrieve(cls, name = ''):
        """Retrieves object in registry with name

           return object with name or  False if no object by name
        """
        return cls.Names.get(name, None)


class StoriedRegistrar(Registrar):
    """Adds store attribute to Registry instances
    """
    __slots__ = ('store', )

    def __init__(self, store=None, **kw):
        """Initializer method for instance.
           Inherited instance attributes:
           .name

           New instance attributes
           .store = reference to shared data store

        """
        super(StoriedRegistrar, self).__init__(**kw)
        self.changeStore(store=store)

    def changeStore(self, store=None):
        """Replace .store """
        from . import storing
        if store is not None:
            if  not isinstance(store, storing.Store):
                raise ValueError("Not store %s" % store)
        self.store = store







def TestTemperature():
    """           """


    #clear registries
    storing.Store.Clear()
    #deeding.Deed.Clear()
    doing.Doer.Clear()

    store = storing.Store(name = 'Test')

    print("\nTesting Temperature Filter")
    filter = TemperatureSensorFilter(name = 'filterSensorTemp', store = store,
                                     group = 'filter.sensor.temperature', output = 'state.temperature',
                                     input = 'ctd', depth = 'state.depth',
                                     parms = dict(window = 60.0, frac = 0.9, preload = 10.0,
                                                  layer = 40.0, tolerance = 5.0))


    output = store.fetch('state.temperature').update(value = 10.0)
    output = store.fetch('state.depth').update(value = 40.0)
    store.expose()
    filter._expose()

    for k in range(1, 300):
        print("")
        store.advanceStamp(0.125)
        s = 10.0 + 2.0 * math.sin(math.pi * 2.0 * k/300.0)
        input = store.fetch('ctd').update(temperature = s)
        filter.update()
        filter._expose()
        print(s)



def TestSalinity():
    """           """

    #clear registries
    storing.Store.Clear()
    #deeding.Deed.Clear()
    doing.Doer.Clear()

    store = storing.Store(name = 'Test')

    print("\nTesting Salinity Filter")
    filter = SalinitySensorFilter(name = 'filterSensorSalinity', store = store,
                                  group = 'filter.sensor.salinity', output = 'state.salinity',
                                  input = 'ctd.salinity',
                                  parms = dict(window = 60.0, frac = 0.9))

    output = store.fetch('state.salinity').update(value = 32.0)
    store.expose()
    filter._expose()

    for k in range(1, 300):
        print("")
        store.advanceStamp(0.125)
        s = 32.0 + 2.0 * math.sin(math.pi * 2.0 * k/300.0)
        input = store.fetch('ctd.salinity').update(value = s)
        filter.update()
        filter._expose()



def Test():
    """Module Common self test

    """
    pass

if __name__ == "__main__":
    Test()

